#ifndef ONEDINTERP_H
#define ONEDINTERP_H


class Interpolate_1D_Private;	// We wrap the implementation in here mainly
							// so clients don't have to include the C-based rspl headers


//	A simple class to store in->out mapping - to provide a more comfortable interface to Interpolate_1D

class Interpolate_1D_Tuple
{
	public:
	Interpolate_1D_Tuple(double in, double out) : in(in), out(out)
	{
	}
	double in, out;
	friend class Interpolate_1D_Private;
};



// Base class for 1D interpolation using the rspl library from Argyll.

class Interpolate_1D
{
	public:
	Interpolate_1D();
	virtual ~Interpolate_1D();

	// High-level initialisation interface - allows points to be added one at a time.
	// Once you've added all the required points, call Populate().
	virtual void Add(double in, double out);
	virtual void Add(Interpolate_1D_Tuple tuple);
	virtual bool Populate(double smoothing=0.25);

	// Low-level initialisation interface - allows loading an "in" and "out" array of doubles en-masse.
	virtual bool Populate(int count,double *in,double *out,double smoothing=0.25);

	// Interpolate interface - must have called Populate() before using these calls.

	virtual double Interpolate(double input);
	virtual double ReverseInterpolate(double output);

	protected:
	Interpolate_1D_Private *priv;
};

#endif

