#ifndef ARGYLLSUPPORT_H
#define ARGYLLSUPPORT_H

#include "support/progress.h"
#include "support/thread.h"
#include "support/configdb.h"
#include "support/searchpath.h"
#include "imagesource/imagesource_devicen_preview.h"


// Wrapper class around Argyll's predefined papersizes
// Supports custom sizes, and generates a command line argument suitable
// for a printtarg call.

class ArgyllPaperSize
{
	public:
	ArgyllPaperSize(const char *name="A4");
	ArgyllPaperSize(int w,int h);
	int GetWidth();
	int GetHeight();
	char *GetPrinttargArgs();
	private:
	const char *size;
	int width;
	int height;
};


// Wrapper class around Argyll's instruments.
// Generates a command line argument suitable for a printtarg call.

class ArgyllInstrument
{
	public:
	ArgyllInstrument(const char *name="DTP41");
	~ArgyllInstrument();
	char *GetPrinttargArgs();
	protected:
	unsigned int inst;
};


class ArgyllSupport : public SearchPathHandler
{
	public:
	ArgyllSupport(const char *binpath="/usr/local/bin");
	virtual ~ArgyllSupport();
	bool CheckBinPath();
	bool CallArgyll(const char *command,Progress *p=NULL);

	// Static function to return a "long" colorant name
	// from a single- or two-character "short name", as used in a TI1/TI3 file
	static const char *GetColorantLongName(const char *shortname);
	static int GetSupportedInstrumentCount();
	static const char *GetInstrumentName(unsigned int i);
	static const char *GetInstrumentDescription(unsigned int i);
	protected:
	static int thread_entry(Thread &t,void *userdata);
};


#endif

