#include <iostream>

#include <gtk/gtk.h>

#include "support/progresstext.h"
#include "progressbar.h"
#include "imagesource/imagesource.h"
#include "imagesource/imagesource_montage.h"
#include "imagesource/imagesource_modifiedgamma.h"
#include "imagesource/imagesource_scaledensity.h"
#include "imagesource/imagesource_hsweep.h"
#include "imagesource/imagesource_util.h"
#include "imageutils/tiffsaver.h"

#include "lincore.h"
#include "linproject.h"
#include "dropsize.h"

#include "dither.h"

using namespace std;



int main(int argc,char **argv)
{
	gtk_init(&argc,&argv);
//	stp_init();
	if(argc<2)
	{
		cerr << "Usage " << argv[0] << " file.gplin" << endl;
		return(0);
	}
	try
	{
		// Figure out dropsize ratios from gplin file
		LinearizationCore core;
		LinProject proj(core);
		proj.SetFilename(argv[1]);
		proj.Load();

		char *fn=proj.GetTempRoot("DropSize",".ti3");
		RawChannelResponses ti3(fn,0.25);
		free(fn);

		ChannelResponse *c=NULL;

		float gamma=proj.gprinter.tuning.FindFloat("TuningGamma");
		cerr << "Using gamma: " << gamma << endl;
		c=ti3.GetChannel("Light Light Black");
		DropSizeResponse d1(c,proj.gprinter.tuning.FindFloat("DropSizeLimit0"),gamma);

		c=ti3.GetChannel("Light Black");
		DropSizeResponse d2(c,proj.gprinter.tuning.FindFloat("DropSizeLimit1"),gamma);

		c=ti3.GetChannel("Black");
		DropSizeResponse d3(c,proj.gprinter.tuning.FindFloat("DropSizeLimit2"),gamma);

		double L=d1.InputToL(100);
		double gp1=d1.InputToGP(100);
		cerr << "Input: " << 100 << " (" << gp1 << ") -> " << L << endl;
		double gp2=d2.LToGP(L);
		double gp3=d3.LToGP(L);
		cerr << " -> " << gp2 << "  -  Large:Med " << gp3/gp2 << endl;
		cerr << " -> " << gp3 << "  -  Large:Small " << gp3/gp1 << endl;
		DitherDropSizes drops(gp3/gp1,gp3/gp2,1.0);
		DitherDropSizes_Single dropslargeonly(3);
		DitherDropSizes_Adaptive dropsadaptive(d1,d2,d3);

		// Now render a sweep pattern.

		ImageSource *is=NULL;
		char *outfn=NULL;

		if(argc==4)
		{
			is=ISLoadImage(argv[2]);
			outfn=argv[3];
		}
		else
		{
			ImageSource_Montage *mon=new ImageSource_Montage(IS_TYPE_CMYK);
			int xres,yres;
			stp_describe_resolution(proj.gprinter.stpvars,&xres,&yres);
			mon->xres=xres;
			mon->yres=yres;

			ISDeviceNValue left(4,0);
			ISDeviceNValue right(4,0);
			right[3]=IS_SAMPLEMAX;

			is=new ImageSource_HSweep(xres*4,yres/4,left,right,IS_TYPE_CMYK);

			is=new ImageSource_ModifiedGamma(is,proj.gprinter.tuning.FindFloat("TuningGamma"));

			ISDeviceNValue densities(4,0);
			densities[3]=IS_SAMPLEMAX*proj.gprinter.tuning.FindFloat("DropSizeLimit2");
			is=new ImageSource_ScaleDensity(is,densities);
			is=new ImageSource_DropSizeDither(is,drops);

			mon->Add(is,0,0);


			is=new ImageSource_HSweep(xres*4,yres/4,left,right,IS_TYPE_CMYK);
			is=new ImageSource_ModifiedGamma(is,proj.gprinter.tuning.FindFloat("TuningGamma"));
			is=new ImageSource_ScaleDensity(is,densities);
			is=new ImageSource_DropSizeDither(is,dropsadaptive);

			mon->Add(is,0,yres/4);


			is=new ImageSource_HSweep(xres*4,yres/4,left,right,IS_TYPE_CMYK);
			is=new ImageSource_ModifiedGamma(is,proj.gprinter.tuning.FindFloat("TuningGamma"));
			is=new ImageSource_ScaleDensity(is,densities);
			is=new ImageSource_DropSizeDither(is,dropslargeonly);

			mon->Add(is,0,yres/2);

			// Now create a sweep using just the large dropsize

			is=mon;

			outfn=argv[2];
		}
		if(argc==2)
		{
			if(is)
			{
				stp_set_string_parameter(proj.gprinter.stpvars,"DitherAlgorithm","Predithered");
				ProgressBar p("Printing...",true);

				double limit=proj.gprinter.tuning.FindFloat("DropSizeLimit2");

//				for(double in=0.0;in<limit;in+=0.01)
//					cerr << "In: " << in << " -> " << dropsadaptive.PickDropSize(in) << endl;
				proj.gprinter.SetProgress(&p);
				proj.Print(is,10,10,true);
				proj.gprinter.SetProgress(NULL);
			}
		}
		else if(is)
		{
			TIFFSaver ts(outfn,ImageSource_rp(is));
			ProgressText p;
			ts.SetProgress(&p);
			ts.Save();
		}
		if(is)
			delete is;
	}
	catch(const char *err)
	{
		cerr << "Error: " << err << endl;
	}
	return(0);
}

