#ifndef DROPSIZE_H
#define DROPSIZE_H

#include "channelresponse.h"
#include "imagesource/imagesource_modifiedgamma.h"

// Basic type for dropsize ratios
class DropSizeRatios
{
	public:
	DropSizeRatios() : ratio1(0.0), ratio2(0.0), ratio3(0.0)
	{
	}
	DropSizeRatios(double r1, double r2,double r3)
	{
		ratio1=r1;
		ratio2=r2;
		ratio3=r3;
	}
	double ratio1,ratio2,ratio3;
};


// Wrapper to simplify translation between density- and gamma-corrected input and raw Gutenprint input

class DropSizeResponse
{
	public:
	DropSizeResponse(ChannelResponse *c,double density=1.0,double gamma=0.25) : channel(c),density(density),gamma(gamma)
	{
		lint=channel->GetLabInterp();
		cerr << "Creating dropsize class for: " << c->GetLongName() << ", density: " << density << endl;
	}
	~DropSizeResponse()
	{
	}
	double InputToL(double in)
	{
		LabValue lab=(*lint)[in];
		return(lab.L);
	}
	double InputToGP(double in)
	{
		in=1.0-ImageSource_ModifiedGamma::ModifiedGamma(1.0-(in/100.0),gamma);
		return(in*density);
	}
	double GPToInput(double in)
	{
		in/=density;
		return(100.0*(1.0-ImageSource_ModifiedGamma::InverseModifiedGamma(1.0-in,gamma)));
	}
	double GPToL(double in)
	{
		return(InputToL(GPToInput(in)));
	}
	double LToGP(double L)
	{
		double in=lint->ReverseInterpolate_L(L);
		return(InputToGP(in));
	}
	ChannelResponse *channel;
	protected:
	Interpolate_Lab *lint;
	double density;
	double gamma;
};

#endif

