#include <iostream>
#include <fstream>

#include <gtk/gtk.h>

#include "imagesource/imagesource.h"
#include "imagesource/imagesource_modifiedgamma.h"
#include "imagesource/imagesource_scaledensity.h"
#include "imagesource/imagesource_devicen_remap.h"
#include "imagesource/imagesource_devicen_preview.h"
#include "imageutils/tiffsaver.h"

#include "imagesource_dropsizesegment.h"

#include "support/progresstext.h"

#include "lincore.h"
#include "linproject.h"
#include "gpdevicensupport.h"
#include "channelresponse.h"
#include "testpatterns.h"
#include "dropsize.h"

using namespace std;


int main(int argc,char **argv)
{
//	stp_init();
	gtk_init(&argc,&argv);
	if(argc>1)
	{
		try
		{

			LinearizationCore core;
			LinProject proj(core);
			proj.SetFilename(argv[1]);
			proj.Load();

			char *fn=proj.GetTempRoot("DropSize",".ti3");
			RawChannelResponses ti3(fn,0.25);
			free(fn);

			ChannelResponse *c=NULL;

			float gamma=proj.gprinter.tuning.FindFloat("TuningGamma");
			cerr << "Using gamma: " << gamma << endl;
			c=ti3.GetChannel("Light Light Black");
			DropSizeResponse d1(c,proj.gprinter.tuning.FindFloat("DropSizeLimit0"),gamma);

			c=ti3.GetChannel("Light Black");
			DropSizeResponse d2(c,proj.gprinter.tuning.FindFloat("DropSizeLimit1"),gamma);

			c=ti3.GetChannel("Black");
			DropSizeResponse d3(c,proj.gprinter.tuning.FindFloat("DropSizeLimit2"),gamma);

			for(double input=5;input<=101.0;input+=5.0)
			{
				double L=d1.InputToL(input);
				double gp1=d1.InputToGP(input);
				cerr << "Input: " << input << " (" << gp1 << ") -> " << L << endl;
				double gp2=d2.LToGP(L);
				cerr << " -> " << gp2 << "  -  Ratio 1: " << gp2/gp1 << endl;
				double gp3=d3.LToGP(L);
				cerr << " -> " << gp3 << "  -  Ratio 2: " << gp3/gp1 << endl;
			}

#if 0
			char *targetsrc=core.targetpath.SearchPaths("DropSize.ti1");
			if(targetsrc)
			{
				cerr << "Found: " << targetsrc << endl;
				char *outfn=proj.GetTempRoot("DropSize",".ti1");
				cerr << "Output filename: " << outfn << endl;

				ifstream in(targetsrc);
				ofstream out(outfn);
				out << in.rdbuf();
				free(outfn);

				char *root=proj.GetTempRoot("DropSize");
				ArgyllPaperSize paper;
				ArgyllInstrument inst;
				Argyll_TestChart_TIFFs *tiff=core.RenderTIFFs(root,paper,inst,180,20,NULL);
				delete tiff;
				free(root);
				free(targetsrc);
			}
			else
				cerr << "Not found" << endl;
#endif

#if 0
			DropSize_Colorants colorants(&proj.gprinter);

			ISDeviceNValue densities(3);
			densities[0]=IS_SAMPLEMAX;
			densities[1]=IS_SAMPLEMAX/2;
			densities[2]=IS_SAMPLEMAX/2;

			ImageSource *is=MakeDensityEvalSweep(colorants,600,200,1,200);

			is=new ImageSource_ModifiedGamma(is,0.25);
			is=new ImageSource_ScaleDensity(is,densities);

			// Now convert to drop-size-segnmented...
			is=new ImageSource_DropSizeSegment(is);

			// Remap to printer DeviceN
			GPColorantList gpcolorants(&proj.gprinter);
			int colcount=gpcolorants.GetColorantCount();
			int blackidx=gpcolorants.GetColorantIndex("Black");
			if(blackidx<0)
				blackidx=gpcolorants.GetColorantIndex("Photo Black");
			cerr << "Got black index:" << blackidx << endl;
			int maptable[1]={0};
			maptable[0]=blackidx;

			is=new ImageSource_DeviceN_Remap(is,maptable,colcount);

#if 1
			proj.gprinter.GPrinter::Print(is,10,10);
			delete is;
#else
			is=new ImageSource_DeviceN_Preview(is,&gpcolorants);

			const char *outfn;
			if(argc>2)
				outfn=argv[2];
			else
				outfn="out.tif";

			TIFFSaver ts(outfn,ImageSource_rp(is));
			ProgressText p;
			ts.SetProgress(&p);
			ts.Save();
#endif
#endif
		}
		catch (const char *err)
		{
			cerr << "Error: " << err << endl;
		}
	}
	return(0);
}

