#include <iostream>

#include <gtk/gtk.h>
#include <gutenprint/gutenprint.h>

#include "config.h"

#include "support/progresstext.h"
#include "progressbar.h"
#include "support/pathsupport.h"
#include "support/util.h"

#include "miscwidgets/generaldialogs.h"

#include "lincore.h"
#include "linproject.h"
#include "projectgui.h"

#include "stepwidget_filename.h"
#include "stepwidget_printsetup.h"
#include "stepwidget_basedensity.h"
#include "stepwidget_basegamma.h"
#include "stepwidget_generatecharts.h"
#include "stepwidget_printcharts.h"
#include "stepwidget_channelinklimits.h"
#include "stepwidget_subchannel.h"
#include "stepwidget_totalinklimit.h"

#include "gettext.h"
#define _(x) gettext(x)


using namespace std;


#define DEFAULT_WIDTH 400
#define DEFAULT_HEIGHT 400


int main(int argc,char **argv)
{
	gtk_init(&argc,&argv);

	const char *filename=NULL;
	if(argc>1)
		filename=argv[1];

	LinearizationCore core;
	ProjectGUI project(core,filename);

	class PrinterDefaults : public PrinterDefaultsCallback
	{
		public:
		PrinterDefaults(LinProject &proj) : PrinterDefaultsCallback(proj) {}
		void SetPrinterDefaults()
		{
			// Default to Raw colour correction
			stp_set_string_parameter(project.gprinter.stpvars,"ColorCorrection","Raw");
		}
	} defaultcallback(project);

	project.SetPrinterDefaultsCallback(&defaultcallback);
	project.SetPrinterDefaults();

	StepSelector_StepList steplist;
	new LinGUIStep_Filename(steplist,_("Choose filename..."),&project);
	new LinGUIStep_PrintSetup(steplist,_("Printer settings..."),&project);
	new LinGUIStep_BaseDensity(steplist,_("Base Density..."),&project);
	new LinGUIStep_BaseGamma(steplist,_("Base Gamma..."),&project);
	new LinGUIStep_GenerateChart(steplist,_("Generate chart(s)..."),&project);
	new LinGUIStep_PrintCharts(steplist,_("Print chart(s)..."),&project);
	new LinGUIStep_ChannelInkLimits(steplist,_("Channel Ink Limits..."),&project);
	new LinGUIStep_SubChannel(steplist,_("Subchannel Ratios..."),&project);
	new LinGUIStep_TotalInkLimit(steplist,_("Total Ink Limit..."),&project);
	project.SetSteps(&steplist);

	if(argc>2)
		project.Import(argv[2]);

	if(!core.CheckBinPath())
		ErrorMessage_Dialog(_("Argyll utilities not found!\nPlease use the Preferences dialog to locate them."));

	gtk_main();
	return(0);
}

