#include <iostream>
#include <math.h>

#include <gtk/gtk.h>
#include <cairo.h>

#include "imagesource_devicen_preview.h"
#include "imagesource_modifiedgamma.h"
#include "linproject.h"
#include "channelresponse.h"

using namespace std;


class Payload
{
	public:
	Payload(RawChannelResponses &ti3,double gamma) : ti3(ti3),gamma(gamma)
	{
	}
	~Payload()
	{
	}
	RawChannelResponses &ti3;
	Interpolate_Lab *smoothed;
	double gamma;
};


static double ForwardGamma(double in,double gamma)
{
	return(1.0-ImageSource_ModifiedGamma::ModifiedGamma(1.0-in,gamma));
}


static void paint_input_vs_dimreturn (GtkWidget *widget,GdkEventExpose *eev,gpointer userdata)
{
	Payload *payload=(Payload *)userdata;
	RawChannelResponses *ti3=&payload->ti3;

	int width  = widget->allocation.width;
	int height = widget->allocation.height;

	cairo_t *cr = gdk_cairo_create (widget->window);
 
	cairo_set_source_rgb (cr, 1,1,1);
	cairo_paint (cr);

	cairo_set_source_rgb (cr, 0,0,0);
	cairo_set_line_width(cr,1.0);
	cairo_move_to (cr,0,0);
	cairo_line_to(cr,0,height-1);
	cairo_line_to(cr,width-1,height-1);
	cairo_stroke (cr);

    cairo_select_font_face (cr, "Sans", CAIRO_FONT_SLANT_ITALIC,
                                        CAIRO_FONT_WEIGHT_BOLD);

	cairo_set_font_size (cr, 15);
	cairo_move_to (cr, width-60, height-5);
	cairo_show_text (cr, "Input");

	ChannelResponse *c=ti3->FirstChannel();
	while(c)
	{
		DeviceNColorant *col=(*ti3)[c->GetChannelIndex()];
		cairo_set_source_rgb (cr, col->red/255.0,col->green/255.0,col->blue/255.0);
//		cerr << "Painting: " << col->GetName() << ": " << col->red << " " << col->green << " " << col->blue << endl;

		int sc=c->GetSampleCount();
		ChannelResponseSample *sam=c->GetSamples();
		Interpolate_Lab *labint=c->GetLabInterp_Smoothed();
		LabValue origin(sam[0]);
		double dL=(*labint)[0.0].L-(*labint)[100.0].L;

		bool first=true;

		for(int i=1;i<sc-1;++i)
		{
			double ci=sam[i+1].input;
			double pi=sam[i-1].input;
			LabValue prev=(*labint)[sam[i-1].input];
			LabValue current=(*labint)[sam[i+1].input];

			double d1=(current.L-origin.L)/(prev.L-origin.L);
			double d2=ForwardGamma(ci/100.0,payload->gamma)/ForwardGamma(pi/100.0,payload->gamma);

			cerr << "D1: " << d1 << ", D2: " << d2 << endl;

			double dimreturn=(d1/d2)/3.0;

			if(first)
			{
				cairo_move_to(cr,(i*width)/sc,height-(dimreturn*height)-1);
				first=false;
			}
			else
				cairo_line_to(cr,(i*width)/sc,height-(dimreturn*height)-1);
			cairo_rel_line_to(cr,2,2);
			cairo_rel_move_to(cr,0,-2);
			cairo_rel_line_to(cr,-2,2);
			cairo_line_to(cr,(i*width)/sc,height-(dimreturn*height)-1);
		}
		c=c->NextChannel();
		cairo_stroke(cr);
	}
	cairo_destroy (cr);
}



static void paint_input_vs_dLbydi (GtkWidget *widget,GdkEventExpose *eev,gpointer userdata)
{
	Payload *payload=(Payload *)userdata;
	RawChannelResponses *ti3=&payload->ti3;

	int width  = widget->allocation.width;
	int height = widget->allocation.height;

	cairo_t *cr = gdk_cairo_create (widget->window);
 
	cairo_set_source_rgb (cr, 1,1,1);
	cairo_paint (cr);

	cairo_set_source_rgb (cr, 0,0,0);
	cairo_set_line_width(cr,1.0);
	cairo_move_to (cr,0,0);
	cairo_line_to(cr,0,height-1);
	cairo_line_to(cr,width-1,height-1);
	cairo_stroke (cr);

    cairo_select_font_face (cr, "Sans", CAIRO_FONT_SLANT_ITALIC,
                                        CAIRO_FONT_WEIGHT_BOLD);

	cairo_set_font_size (cr, 15);
	cairo_move_to (cr, width-60, height-5);
	cairo_show_text (cr, "Input");

	ChannelResponse *c=ti3->FirstChannel();
	while(c)
	{
		DeviceNColorant *col=(*ti3)[c->GetChannelIndex()];
		cairo_set_source_rgb (cr, col->red/255.0,col->green/255.0,col->blue/255.0);
//		cerr << "Painting: " << col->GetName() << ": " << col->red << " " << col->green << " " << col->blue << endl;

		int sc=c->GetSampleCount();
		ChannelResponseSample *sam=c->GetSamples();
		Interpolate_Lab *labint=c->GetLabInterp_Smoothed();
		LabValue origin(sam[0]);
		double dL=(*labint)[0.0].L-(*labint)[100.0].L;

		bool first=true;

		for(int i=1;i<sc-2;++i)
		{
			double ci=sam[i+1].input;
			double pi=sam[i-1].input;
			LabValue prev=(*labint)[sam[i-1].input];
			LabValue current=(*labint)[sam[i+1].input];

			double d3=prev.L-current.L;
			double di=ForwardGamma(ci/100.0,payload->gamma)-ForwardGamma(pi/100.0,payload->gamma);
			double dLbydi=(d3/di)/dL;

			if(first)
			{
				cairo_move_to(cr,(i*width)/sc,height-(dLbydi*height)/10.0-1);
				first=false;
			}
			else
				cairo_line_to(cr,(i*width)/sc,height-(dLbydi*height)/10.0-1);
			cairo_rel_line_to(cr,2,2);
			cairo_rel_move_to(cr,0,-2);
			cairo_rel_line_to(cr,-2,2);
			cairo_line_to(cr,(i*width)/sc,height-(dLbydi*height)/10.0-1);
		}
		c=c->NextChannel();
		cairo_stroke(cr);
	}
	cairo_destroy (cr);
}


static void paint_input_vs_dEbydi (GtkWidget *widget,GdkEventExpose *eev,gpointer userdata)
{
	Payload *payload=(Payload *)userdata;
	RawChannelResponses *ti3=&payload->ti3;

	int width  = widget->allocation.width;
	int height = widget->allocation.height;

	cairo_t *cr = gdk_cairo_create (widget->window);
 
	cairo_set_source_rgb (cr, 1,1,1);
	cairo_paint (cr);

	cairo_set_source_rgb (cr, 0,0,0);
	cairo_set_line_width(cr,1.0);
	cairo_move_to (cr,0,0);
	cairo_line_to(cr,0,height-1);
	cairo_line_to(cr,width-1,height-1);
	cairo_stroke (cr);

    cairo_select_font_face (cr, "Sans", CAIRO_FONT_SLANT_ITALIC,
                                        CAIRO_FONT_WEIGHT_BOLD);

	cairo_set_font_size (cr, 15);
	cairo_move_to (cr, width-60, height-5);
	cairo_show_text (cr, "Input");

	ChannelResponse *c=ti3->FirstChannel();
	while(c)
	{
		DeviceNColorant *col=(*ti3)[c->GetChannelIndex()];
		cairo_set_source_rgb (cr, col->red/255.0,col->green/255.0,col->blue/255.0);
//		cerr << "Painting: " << col->GetName() << ": " << col->red << " " << col->green << " " << col->blue << endl;

		int sc=c->GetSampleCount();
		ChannelResponseSample *sam=c->GetSamples();
		Interpolate_Lab *labint=c->GetLabInterp_Smoothed();
		LabValue origin(sam[0]);
		double dL=(*labint)[0.0].L-(*labint)[100.0].L;
		cairo_move_to(cr,0,1);
		float slope=1.0;
		bool first=true;
		for(int i=1;i<sc-2;++i)
		{
			double ci=sam[i+1].input;
			double pi=sam[i-1].input;
			LabValue prev=(*labint)[sam[i-1].input];
			LabValue current=(*labint)[sam[i+1].input];

			double d3=current.dE(prev);
			double di=ForwardGamma(ci/100.0,payload->gamma)-ForwardGamma(pi/100.0,payload->gamma);
			double dEbydi=d3/di;

			if(first)
			{
				cairo_move_to(cr,(i*width)/sc,height-(dEbydi*height)/750.0-1);
				first=false;
			}
			else
				cairo_line_to(cr,(i*width)/sc,height-(dEbydi*height)/750.0-1);
	
			cairo_rel_line_to(cr,2,2);
			cairo_rel_move_to(cr,0,-2);
			cairo_rel_line_to(cr,-2,2);
			cairo_line_to(cr,(i*width)/sc,height-(dEbydi*height)/750.0-1);
		}
		c=c->NextChannel();
		cairo_stroke(cr);
	}
	cairo_destroy (cr);
}


// Hue trueness measure


static void paint_input_vs_huetrueness (GtkWidget *widget,GdkEventExpose *eev,gpointer userdata)
{
	RawChannelResponses *ti3=(RawChannelResponses *)userdata;

	int width  = widget->allocation.width;
	int height = widget->allocation.height;

	cairo_t *cr = gdk_cairo_create (widget->window);
 
	cairo_set_source_rgb (cr, 1,1,1);
	cairo_paint (cr);

	cairo_set_source_rgb (cr, 0,0,0);
	cairo_set_line_width(cr,1.0);
	cairo_move_to (cr,0,0);
	cairo_line_to(cr,0,height-1);
	cairo_line_to(cr,width-1,height-1);
	cairo_stroke (cr);

    cairo_select_font_face (cr, "Sans", CAIRO_FONT_SLANT_ITALIC,
                                        CAIRO_FONT_WEIGHT_BOLD);

	cairo_set_font_size (cr, 15);
	cairo_move_to (cr, width-60, height-5);
	cairo_show_text (cr, "Input");

	ChannelResponse *c=ti3->FirstChannel();
	while(c)
	{
		DeviceNColorant *col=(*ti3)[c->GetChannelIndex()];
		cairo_set_source_rgb (cr, col->red/255.0,col->green/255.0,col->blue/255.0);
//		cerr << "Painting: " << col->GetName() << ": " << col->red << " " << col->green << " " << col->blue << endl;

		int sc=c->GetSampleCount();
		ChannelResponseSample *sam=c->GetSamples();
		Interpolate_Lab *labint=c->GetLabInterp_Smoothed();
		LabValue origin(sam[0]);
		cairo_move_to(cr,0,1);
		float slope=1.0;
		for(int i=2;i<sc-2;++i)
		{
			LabValue prev=(*labint)[sam[i-1].input];
			LabValue current=(*labint)[sam[i+1].input];
//			LabValue prev(sam[i-2]);
//			LabValue current(sam[i+2]);

			float d1=(prev-origin).Magnitude();
			float d2=(current-origin).Magnitude();
			float d3=(current-prev).Magnitude();

			slope=(d2-d1)/d3;

			cairo_line_to(cr,(i*width)/sc,height-(slope*height)-1);
			cairo_rel_line_to(cr,2,2);
			cairo_rel_move_to(cr,0,-2);
			cairo_rel_line_to(cr,-2,2);
			cairo_line_to(cr,(i*width)/sc,height-(slope*height)-1);
		}
		c=c->NextChannel();
		cairo_stroke(cr);
	}
	cairo_destroy (cr);
}


// a/b plane


static void paint_a_b_plane (GtkWidget *widget,GdkEventExpose *eev,gpointer userdata)
{
	RawChannelResponses *ti3=(RawChannelResponses *)userdata;

	int width  = widget->allocation.width;
	int height = widget->allocation.height;

	cairo_t *cr = gdk_cairo_create (widget->window);
 
	cairo_set_source_rgb (cr, 1,1,1);
	cairo_paint (cr);

	cairo_set_source_rgb (cr, 0,0,0);
	cairo_set_line_width(cr,1.0);
	cairo_move_to (cr, width/2,0);
	cairo_line_to(cr,width/2,height);
	cairo_move_to (cr, 0, height/2);
	cairo_line_to(cr,width,height/2);
	cairo_stroke (cr);

    cairo_select_font_face (cr, "Sans", CAIRO_FONT_SLANT_ITALIC,
                                        CAIRO_FONT_WEIGHT_BOLD);
	cairo_set_font_size (cr, 9);
	cairo_move_to (cr, width/2+5, 12);
	cairo_show_text (cr, "+128");
	cairo_move_to (cr, width/2+5, height-3);
	cairo_show_text (cr, "-128");

	cairo_move_to (cr, 5, height/2+15);
	cairo_show_text (cr, "-128");
	cairo_move_to (cr, width-30, height/2+15);
	cairo_show_text (cr, "+128");

	cairo_set_font_size (cr, 15);
	cairo_move_to (cr, width/2+50, 20);
	cairo_show_text (cr, "b");
	cairo_move_to (cr, width-20, height/2+35);
	cairo_show_text (cr, "a");

	ChannelResponse *c=ti3->FirstChannel();
	while(c)
	{
		DeviceNColorant *col=(*ti3)[c->GetChannelIndex()];
		cairo_set_source_rgb (cr, col->red/255.0,col->green/255.0,col->blue/255.0);
//		cerr << "Painting: " << col->GetName() << ": " << col->red << " " << col->green << " " << col->blue << endl;

		int sc=c->GetSampleCount();
		ChannelResponseSample *sam=c->GetSamples();
		for(int i=0;i<sc;++i)
		{
			LabValue l(sam[i]);
			cairo_move_to(cr,width/2+(l.a*width)/256-1,height/2-(l.b*height)/256-1);
			cairo_rel_line_to(cr,2,2);
			cairo_rel_move_to(cr,0,-2);
			cairo_rel_line_to(cr,-2,2);
			cairo_line_to(cr,width/2+(l.a*width)/256-1,height/2-(l.b*height)/256-1);
		}
		c=c->NextChannel();
		cairo_stroke(cr);
	}
	cairo_destroy (cr);
}


static void paint_input_vs_c (GtkWidget *widget,GdkEventExpose *eev,gpointer userdata)
{
	RawChannelResponses *ti3=(RawChannelResponses *)userdata;

	int width  = widget->allocation.width;
	int height = widget->allocation.height;

	cairo_t *cr = gdk_cairo_create (widget->window);
 
	cairo_set_source_rgb (cr, 1,1,1);
	cairo_paint (cr);

	cairo_set_source_rgb (cr, 0,0,0);
	cairo_set_line_width(cr,1.0);
	cairo_move_to (cr,0,0);
	cairo_line_to(cr,0,height-1);
	cairo_line_to(cr,width-1,height-1);
	cairo_stroke (cr);

    cairo_select_font_face (cr, "Sans", CAIRO_FONT_SLANT_ITALIC,
                                        CAIRO_FONT_WEIGHT_BOLD);

	cairo_set_font_size (cr, 15);
	cairo_move_to (cr,15, 12);
	cairo_show_text (cr, "c");
	cairo_move_to (cr, width-60, height-5);
	cairo_show_text (cr, "Input");

	ChannelResponse *c=ti3->FirstChannel();
	while(c)
	{
		DeviceNColorant *col=(*ti3)[c->GetChannelIndex()];
		cairo_set_source_rgb (cr, col->red/255.0,col->green/255.0,col->blue/255.0);
//		cerr << "Painting: " << col->GetName() << ": " << col->red << " " << col->green << " " << col->blue << endl;

		int sc=c->GetSampleCount();
		ChannelResponseSample *sam=c->GetSamples();
		cairo_move_to(cr,0,height-1);
		for(int i=0;i<sc;++i)
		{
			float input=sam[i].input;
			LabValue l(sam[i]);
			float c=sqrt(l.a*l.a+l.b*l.b);
			cairo_line_to(cr,(input*width)/100,height-(c*height)/160-1);
			cairo_rel_line_to(cr,2,2);
			cairo_rel_move_to(cr,0,-2);
			cairo_rel_line_to(cr,-2,2);
			cairo_move_to(cr,(input*width)/100,height-(c*height)/160-1);
		}
		c=c->NextChannel();
		cairo_stroke(cr);
	}
	cairo_destroy (cr);
}


static void paint_input_vs_c_linearized (GtkWidget *widget,GdkEventExpose *eev,gpointer userdata)
{
	RawChannelResponses *ti3=(RawChannelResponses *)userdata;

	int width  = widget->allocation.width;
	int height = widget->allocation.height;

	cairo_t *cr = gdk_cairo_create (widget->window);
 
	cairo_set_source_rgb (cr, 1,1,1);
	cairo_paint (cr);

	cairo_set_source_rgb (cr, 0,0,0);
	cairo_set_line_width(cr,1.0);
	cairo_move_to (cr,0,0);
	cairo_line_to(cr,0,height-1);
	cairo_line_to(cr,width-1,height-1);
	cairo_stroke (cr);

    cairo_select_font_face (cr, "Sans", CAIRO_FONT_SLANT_ITALIC,
                                        CAIRO_FONT_WEIGHT_BOLD);

	cairo_set_font_size (cr, 15);
	cairo_move_to (cr,15, 12);
	cairo_show_text (cr, "c");
	cairo_move_to (cr, width-60, height-5);
	cairo_show_text (cr, "Input");

	ChannelResponse *c=ti3->FirstChannel();
	while(c)
	{
		DeviceNColorant *col=(*ti3)[c->GetChannelIndex()];
		cairo_set_source_rgb (cr, col->red/255.0,col->green/255.0,col->blue/255.0);
//		cerr << "Painting: " << col->GetName() << ": " << col->red << " " << col->green << " " << col->blue << endl;

		int sc=c->GetSampleCount();
		ChannelResponseSample *sam=c->GetSamples();
		cairo_move_to(cr,0,height-1);
		for(int i=0;i<sc;++i)
		{
			float input=sam[i].input;
			LabValue l(sam[i]);
			input=c->Interpolate(input);
			float c=sqrt(l.a*l.a+l.b*l.b);
			cairo_line_to(cr,(input*width)/100,height-(c*height)/160-1);
			cairo_rel_line_to(cr,2,2);
			cairo_rel_move_to(cr,0,-2);
			cairo_rel_line_to(cr,-2,2);
			cairo_move_to(cr,(input*width)/100,height-(c*height)/160-1);
		}
		c=c->NextChannel();
		cairo_stroke(cr);
	}
	cairo_destroy (cr);
}


static void paint_input_vs_Lstar_Raw (GtkWidget *widget,GdkEventExpose *eev,gpointer userdata)
{
	Payload *payload=(Payload *)userdata;
	RawChannelResponses *ti3=&payload->ti3;
	ChannelResponseSample *paper=ti3->GetPaper();
	LabValue paperlab(*paper);

	int width  = widget->allocation.width;
	int height = widget->allocation.height;

	cairo_t *cr = gdk_cairo_create (widget->window);
 
	cairo_set_source_rgb (cr, 1,1,1);
	cairo_paint (cr);

	cairo_set_source_rgb (cr, 0,0,0);
	cairo_set_line_width(cr,1.0);
	cairo_move_to (cr, 1,0);
	cairo_line_to(cr,1,height);
	cairo_move_to(cr,0,height-1);
	cairo_line_to (cr, width,height-1);
	cairo_stroke (cr);

    cairo_select_font_face (cr, "Sans", CAIRO_FONT_SLANT_ITALIC,
                                        CAIRO_FONT_WEIGHT_BOLD);
	cairo_set_font_size (cr, 9);
	cairo_move_to (cr, 5, 12);
	cairo_show_text (cr, "L* 100");
	cairo_move_to (cr, width-75, height-3);
	cairo_show_text (cr, "Input 100%");

	ChannelResponse *c=ti3->FirstChannel();
	while(c)
	{
		DeviceNColorant *col=(*ti3)[c->GetChannelIndex()];
		cairo_set_source_rgb (cr, col->red/255.0,col->green/255.0,col->blue/255.0);

		int sc=c->GetSampleCount();
		ChannelResponseSample *sam=c->GetSamples();

		float lmin=100.0;
		float lmax=0.0;
		for(int i=0;i<sc;++i)
		{
			LabValue l(sam[i]);
			if(l.L<lmin)
				lmin=l.L;
			if(l.L>lmax)
				lmax=l.L;
		}

		cairo_move_to(cr,0,height-(paperlab.L*height)/100);
		for(int i=0;i<sc;++i)
		{
			float input=sam[i].input;
			LabValue l(sam[i]);

			input=100.0 * ForwardGamma(input/100.0,payload->gamma);
			
			cairo_line_to(cr,(input*width)/100.0,height-(l.L*height)/100.0);
			cairo_rel_line_to(cr,2,2);
			cairo_rel_move_to(cr,0,-2);
			cairo_rel_line_to(cr,-2,2);
			cairo_move_to(cr,(input*width)/100.0,height-(l.L*height)/100);
		}

		c=c->NextChannel();
		cairo_stroke(cr);
	}
	cairo_destroy (cr);
}


static void paint_input_vs_Lstar_Gamma (GtkWidget *widget,GdkEventExpose *eev,gpointer userdata)
{
	RawChannelResponses *ti3=(RawChannelResponses *)userdata;
	ChannelResponseSample *paper=ti3->GetPaper();
	LabValue paperlab(*paper);

	int width  = widget->allocation.width;
	int height = widget->allocation.height;

	cairo_t *cr = gdk_cairo_create (widget->window);
 
	cairo_set_source_rgb (cr, 1,1,1);
	cairo_paint (cr);

	cairo_set_source_rgb (cr, 0,0,0);
	cairo_set_line_width(cr,1.0);
	cairo_move_to (cr, 1,0);
	cairo_line_to(cr,1,height);
	cairo_move_to(cr,0,height-1);
	cairo_line_to (cr, width,height-1);
	cairo_stroke (cr);

    cairo_select_font_face (cr, "Sans", CAIRO_FONT_SLANT_ITALIC,
                                        CAIRO_FONT_WEIGHT_BOLD);
	cairo_set_font_size (cr, 9);
	cairo_move_to (cr, 5, 12);
	cairo_show_text (cr, "L* 100");
	cairo_move_to (cr, width-75, height-3);
	cairo_show_text (cr, "Input 100%");

	ChannelResponse *c=ti3->FirstChannel();
	while(c)
	{
		DeviceNColorant *col=(*ti3)[c->GetChannelIndex()];
		cairo_set_source_rgb (cr, col->red/255.0,col->green/255.0,col->blue/255.0);

		int sc=c->GetSampleCount();
		ChannelResponseSample *sam=c->GetSamples();

		float lmin=100.0;
		float lmax=0.0;
		for(int i=0;i<sc;++i)
		{
			LabValue l(sam[i]);
			if(l.L<lmin)
				lmin=l.L;
			if(l.L>lmax)
				lmax=l.L;
		}

		cairo_move_to(cr,0,height-(paperlab.L*height)/100);
		for(int i=0;i<sc;++i)
		{
			float input=sam[i].input;
			LabValue l(sam[i]);
			
			cairo_line_to(cr,(input*width)/100.0,height-(l.L*height)/100.0);
			cairo_rel_line_to(cr,2,2);
			cairo_rel_move_to(cr,0,-2);
			cairo_rel_line_to(cr,-2,2);
			cairo_move_to(cr,(input*width)/100.0,height-(l.L*height)/100);
		}

		c=c->NextChannel();
		cairo_stroke(cr);
	}
	cairo_destroy (cr);
}


static void paint_input_vs_Lstar_linearized (GtkWidget *widget,GdkEventExpose *eev,gpointer userdata)
{
	RawChannelResponses *ti3=(RawChannelResponses *)userdata;
	ChannelResponseSample *paper=ti3->GetPaper();
	LabValue paperlab(*paper);

	int width  = widget->allocation.width;
	int height = widget->allocation.height;

	cairo_t *cr = gdk_cairo_create (widget->window);
 
	cairo_set_source_rgb (cr, 1,1,1);
	cairo_paint (cr);

	cairo_set_source_rgb (cr, 0,0,0);
	cairo_set_line_width(cr,1.0);
	cairo_move_to (cr, 1,0);
	cairo_line_to(cr,1,height);
	cairo_move_to(cr,0,height-1);
	cairo_line_to (cr, width,height-1);
	cairo_stroke (cr);

    cairo_select_font_face (cr, "Sans", CAIRO_FONT_SLANT_ITALIC,
                                        CAIRO_FONT_WEIGHT_BOLD);
	cairo_set_font_size (cr, 9);
	cairo_move_to (cr, 5, 12);
	cairo_show_text (cr, "L* 100");
	cairo_move_to (cr, width-75, height-3);
	cairo_show_text (cr, "Input 100%");

	ChannelResponse *c=ti3->FirstChannel();
	while(c)
	{
		DeviceNColorant *col=(*ti3)[c->GetChannelIndex()];
		cairo_set_source_rgb (cr, col->red/255.0,col->green/255.0,col->blue/255.0);

		int sc=c->GetSampleCount();
		ChannelResponseSample *sam=c->GetSamples();

		double inklimit=c->GetInkLimit();
		Interpolate_Lab *labinterp=c->GetLabInterp();
		LabValue lmax=(*labinterp)[0.0];
		LabValue lmin=(*labinterp)[inklimit];

		cairo_move_to(cr,0,height-(paperlab.L*height)/100);
		for(int i=0;i<sc;++i)
		{
			float input=sam[i].input;
			if(input<=inklimit)
			{
				LabValue l(sam[i]);
				if(c->GetType()==CHANNELRESPONSETYPE_YELLOW)
					input=inklimit*c->Interpolate(input)/100.0;
				else
					input=inklimit*c->Interpolate(input)/100.0;
				
				cairo_move_to(cr,(input*width)/100.0,height-(l.L*height)/100.0);
				cairo_rel_line_to(cr,2,2);
				cairo_rel_move_to(cr,0,-2);
				cairo_rel_line_to(cr,-2,2);
				cairo_line_to(cr,(input*width)/100.0,height-(l.L*height)/100.0);
			}
		}

		cairo_move_to(cr,0,height-(height*lmax.L)/100.0);
		cairo_line_to(cr,(inklimit*width)/100.0-1,height-(height*lmin.L)/100.0);

		c=c->NextChannel();
		cairo_stroke(cr);
	}
	cairo_destroy (cr);
}


static void paint_input_vs_a_raw (GtkWidget *widget,GdkEventExpose *eev,gpointer userdata)
{
	Payload *payload=(Payload *)userdata;
	RawChannelResponses *ti3=&payload->ti3;
	ChannelResponseSample *paper=ti3->GetPaper();
	LabValue paperlab(*paper);

	int width  = widget->allocation.width;
	int height = widget->allocation.height;

	cairo_t *cr = gdk_cairo_create (widget->window);
 
	cairo_set_source_rgb (cr, 1,1,1);
	cairo_paint (cr);

	cairo_set_source_rgb (cr, 0,0,0);
	cairo_set_line_width(cr,1.0);
	cairo_move_to (cr, 1,0);
	cairo_line_to(cr,1,height);
	cairo_move_to(cr,0,height/2);
	cairo_line_to (cr, width,height/2);
	cairo_stroke (cr);

    cairo_select_font_face (cr, "Sans", CAIRO_FONT_SLANT_ITALIC,
                                        CAIRO_FONT_WEIGHT_BOLD);
	cairo_set_font_size (cr, 9);
	cairo_move_to (cr, 5, 12);
	cairo_show_text (cr, "a 128");
	cairo_move_to (cr, 5, height-12);
	cairo_show_text (cr, "a -128");
	cairo_move_to (cr, width-75, height/2-3);
	cairo_show_text (cr, "Input 100%");

	ChannelResponse *c=ti3->FirstChannel();
	while(c)
	{
		DeviceNColorant *col=(*ti3)[c->GetChannelIndex()];
		cairo_set_source_rgb (cr, col->red/255.0,col->green/255.0,col->blue/255.0);
//		cerr << "Painting: " << col->GetName() << ": " << col->red << " " << col->green << " " << col->blue << endl;

		int sc=c->GetSampleCount();
		ChannelResponseSample *sam=c->GetSamples();
		cairo_move_to(cr,0,height/2-(paperlab.a*height)/256);
		for(int i=0;i<sc;++i)
		{
			float input=sam[i].input;
			LabValue l(sam[i]);

			input=100.0 * ForwardGamma(input/100.0,payload->gamma);

//			cerr << "input " << input << " -> " << l.L << endl;
			cairo_line_to(cr,(input*width)/100.0,height/2-(l.a*height)/256);
			cairo_rel_line_to(cr,2,2);
			cairo_rel_move_to(cr,0,-2);
			cairo_rel_line_to(cr,-2,2);
			cairo_move_to(cr,(input*width)/100.0,height/2-(l.a*height)/256);
		}
		c=c->NextChannel();
		cairo_stroke(cr);
	}
	cairo_destroy (cr);
}


static void paint_input_vs_a (GtkWidget *widget,GdkEventExpose *eev,gpointer userdata)
{
	RawChannelResponses *ti3=(RawChannelResponses *)userdata;
	ChannelResponseSample *paper=ti3->GetPaper();
	LabValue paperlab(*paper);

	int width  = widget->allocation.width;
	int height = widget->allocation.height;

	cairo_t *cr = gdk_cairo_create (widget->window);
 
	cairo_set_source_rgb (cr, 1,1,1);
	cairo_paint (cr);

	cairo_set_source_rgb (cr, 0,0,0);
	cairo_set_line_width(cr,1.0);
	cairo_move_to (cr, 1,0);
	cairo_line_to(cr,1,height);
	cairo_move_to(cr,0,height/2);
	cairo_line_to (cr, width,height/2);
	cairo_stroke (cr);

    cairo_select_font_face (cr, "Sans", CAIRO_FONT_SLANT_ITALIC,
                                        CAIRO_FONT_WEIGHT_BOLD);
	cairo_set_font_size (cr, 9);
	cairo_move_to (cr, 5, 12);
	cairo_show_text (cr, "a 128");
	cairo_move_to (cr, 5, height-12);
	cairo_show_text (cr, "a -128");
	cairo_move_to (cr, width-75, height/2-3);
	cairo_show_text (cr, "Input 100%");

	ChannelResponse *c=ti3->FirstChannel();
	while(c)
	{
		DeviceNColorant *col=(*ti3)[c->GetChannelIndex()];
		cairo_set_source_rgb (cr, col->red/255.0,col->green/255.0,col->blue/255.0);
//		cerr << "Painting: " << col->GetName() << ": " << col->red << " " << col->green << " " << col->blue << endl;

		int sc=c->GetSampleCount();
		ChannelResponseSample *sam=c->GetSamples();
		cairo_move_to(cr,0,height/2-(paperlab.a*height)/256);
		for(int i=0;i<sc;++i)
		{
			float input=sam[i].input;
			LabValue l(sam[i]);

//			cerr << "input " << input << " -> " << l.L << endl;
			cairo_line_to(cr,(input*width)/100.0,height/2-(l.a*height)/256);
			cairo_rel_line_to(cr,2,2);
			cairo_rel_move_to(cr,0,-2);
			cairo_rel_line_to(cr,-2,2);
			cairo_move_to(cr,(input*width)/100.0,height/2-(l.a*height)/256);
		}
		c=c->NextChannel();
		cairo_stroke(cr);
	}
	cairo_destroy (cr);
}


static void paint_input_vs_a_linearized (GtkWidget *widget,GdkEventExpose *eev,gpointer userdata)
{
	RawChannelResponses *ti3=(RawChannelResponses *)userdata;
	ChannelResponseSample *paper=ti3->GetPaper();
	LabValue paperlab(*paper);

	int width  = widget->allocation.width;
	int height = widget->allocation.height;

	cairo_t *cr = gdk_cairo_create (widget->window);
 
	cairo_set_source_rgb (cr, 1,1,1);
	cairo_paint (cr);

	cairo_set_source_rgb (cr, 0,0,0);
	cairo_set_line_width(cr,1.0);
	cairo_move_to (cr, 1,0);
	cairo_line_to(cr,1,height);
	cairo_move_to(cr,0,height/2);
	cairo_line_to (cr, width,height/2);
	cairo_stroke (cr);

    cairo_select_font_face (cr, "Sans", CAIRO_FONT_SLANT_ITALIC,
                                        CAIRO_FONT_WEIGHT_BOLD);
	cairo_set_font_size (cr, 9);
	cairo_move_to (cr, 5, 12);
	cairo_show_text (cr, "a 128");
	cairo_move_to (cr, 5, height-12);
	cairo_show_text (cr, "a -128");
	cairo_move_to (cr, width-75, height/2-3);
	cairo_show_text (cr, "Input 100%");

	ChannelResponse *c=ti3->FirstChannel();
	while(c)
	{
		DeviceNColorant *col=(*ti3)[c->GetChannelIndex()];
		cairo_set_source_rgb (cr, col->red/255.0,col->green/255.0,col->blue/255.0);
//		cerr << "Painting: " << col->GetName() << ": " << col->red << " " << col->green << " " << col->blue << endl;

		int sc=c->GetSampleCount();
		ChannelResponseSample *sam=c->GetSamples();
		cairo_move_to(cr,0,height/2-(paperlab.a*height)/256);
		for(int i=0;i<sc;++i)
		{
			float input=sam[i].input;
			LabValue l(sam[i]);
			input=c->Interpolate(input);

//			cerr << "input " << input << " -> " << l.L << endl;
			cairo_line_to(cr,(input*width)/100.0,height/2-(l.a*height)/256);
			cairo_rel_line_to(cr,2,2);
			cairo_rel_move_to(cr,0,-2);
			cairo_rel_line_to(cr,-2,2);
			cairo_move_to(cr,(input*width)/100.0,height/2-(l.a*height)/256);
		}
		c=c->NextChannel();
		cairo_stroke(cr);
	}
	cairo_destroy (cr);
}


static void paint_input_vs_b_raw (GtkWidget *widget,GdkEventExpose *eev,gpointer userdata)
{
	Payload *payload=(Payload *)userdata;
	RawChannelResponses *ti3=&payload->ti3;
	ChannelResponseSample *paper=ti3->GetPaper();
	LabValue paperlab(*paper);

	int width  = widget->allocation.width;
	int height = widget->allocation.height;

	cairo_t *cr = gdk_cairo_create (widget->window);
 
	cairo_set_source_rgb (cr, 1,1,1);
	cairo_paint (cr);

	cairo_set_source_rgb (cr, 0,0,0);
	cairo_set_line_width(cr,1.0);
	cairo_move_to (cr, 1,0);
	cairo_line_to(cr,1,height);
	cairo_move_to(cr,0,height/2);
	cairo_line_to (cr, width,height/2);
	cairo_stroke (cr);

    cairo_select_font_face (cr, "Sans", CAIRO_FONT_SLANT_ITALIC,
                                        CAIRO_FONT_WEIGHT_BOLD);
	cairo_set_font_size (cr, 9);
	cairo_move_to (cr, 5, 12);
	cairo_show_text (cr, "b 128");
	cairo_move_to (cr, 5, height-12);
	cairo_show_text (cr, "b -128");
	cairo_move_to (cr, width-75, height/2-3);
	cairo_show_text (cr, "Input 100%");

	ChannelResponse *c=ti3->FirstChannel();
	while(c)
	{
		DeviceNColorant *col=(*ti3)[c->GetChannelIndex()];
		cairo_set_source_rgb (cr, col->red/255.0,col->green/255.0,col->blue/255.0);
//		cerr << "Painting: " << col->GetName() << ": " << col->red << " " << col->green << " " << col->blue << endl;

		int sc=c->GetSampleCount();
		ChannelResponseSample *sam=c->GetSamples();
		cairo_move_to(cr,0,height/2-(paperlab.b*height)/256);
		for(int i=0;i<sc;++i)
		{
			float input=sam[i].input;
			LabValue l(sam[i]);

			input=100.0 * ForwardGamma(input/100.0,payload->gamma);

//			cerr << "input " << input << " -> " << l.L << endl;
			cairo_line_to(cr,(input*width)/100.0,height/2-(l.b*height)/256);
			cairo_rel_line_to(cr,2,2);
			cairo_rel_move_to(cr,0,-2);
			cairo_rel_line_to(cr,-2,2);
			cairo_move_to(cr,(input*width)/100.0,height/2-(l.b*height)/256);
		}
		c=c->NextChannel();
		cairo_stroke(cr);
	}
	cairo_destroy (cr);
}


static void paint_input_vs_b (GtkWidget *widget,GdkEventExpose *eev,gpointer userdata)
{
	RawChannelResponses *ti3=(RawChannelResponses *)userdata;
	ChannelResponseSample *paper=ti3->GetPaper();
	LabValue paperlab(*paper);

	int width  = widget->allocation.width;
	int height = widget->allocation.height;

	cairo_t *cr = gdk_cairo_create (widget->window);
 
	cairo_set_source_rgb (cr, 1,1,1);
	cairo_paint (cr);

	cairo_set_source_rgb (cr, 0,0,0);
	cairo_set_line_width(cr,1.0);
	cairo_move_to (cr, 1,0);
	cairo_line_to(cr,1,height);
	cairo_move_to(cr,0,height/2);
	cairo_line_to (cr, width,height/2);
	cairo_stroke (cr);

    cairo_select_font_face (cr, "Sans", CAIRO_FONT_SLANT_ITALIC,
                                        CAIRO_FONT_WEIGHT_BOLD);
	cairo_set_font_size (cr, 9);
	cairo_move_to (cr, 5, 12);
	cairo_show_text (cr, "b 128");
	cairo_move_to (cr, 5, height-12);
	cairo_show_text (cr, "b -128");
	cairo_move_to (cr, width-75, height/2-3);
	cairo_show_text (cr, "Input 100%");

	ChannelResponse *c=ti3->FirstChannel();
	while(c)
	{
		DeviceNColorant *col=(*ti3)[c->GetChannelIndex()];
		cairo_set_source_rgb (cr, col->red/255.0,col->green/255.0,col->blue/255.0);
//		cerr << "Painting: " << col->GetName() << ": " << col->red << " " << col->green << " " << col->blue << endl;

		int sc=c->GetSampleCount();
		ChannelResponseSample *sam=c->GetSamples();
		cairo_move_to(cr,0,height/2-(paperlab.b*height)/256);
		for(int i=0;i<sc;++i)
		{
			float input=sam[i].input;
			LabValue l(sam[i]);

//			cerr << "input " << input << " -> " << l.L << endl;
			cairo_line_to(cr,(input*width)/100.0,height/2-(l.b*height)/256);
			cairo_rel_line_to(cr,2,2);
			cairo_rel_move_to(cr,0,-2);
			cairo_rel_line_to(cr,-2,2);
			cairo_move_to(cr,(input*width)/100.0,height/2-(l.b*height)/256);
		}
		c=c->NextChannel();
		cairo_stroke(cr);
	}
	cairo_destroy (cr);
}


static void paint_input_vs_b_linearized (GtkWidget *widget,GdkEventExpose *eev,gpointer userdata)
{
	RawChannelResponses *ti3=(RawChannelResponses *)userdata;
	ChannelResponseSample *paper=ti3->GetPaper();
	LabValue paperlab(*paper);

	int width  = widget->allocation.width;
	int height = widget->allocation.height;

	cairo_t *cr = gdk_cairo_create (widget->window);
 
	cairo_set_source_rgb (cr, 1,1,1);
	cairo_paint (cr);

	cairo_set_source_rgb (cr, 0,0,0);
	cairo_set_line_width(cr,1.0);
	cairo_move_to (cr, 1,0);
	cairo_line_to(cr,1,height);
	cairo_move_to(cr,0,height/2);
	cairo_line_to (cr, width,height/2);
	cairo_stroke (cr);

    cairo_select_font_face (cr, "Sans", CAIRO_FONT_SLANT_ITALIC,
                                        CAIRO_FONT_WEIGHT_BOLD);
	cairo_set_font_size (cr, 9);
	cairo_move_to (cr, 5, 12);
	cairo_show_text (cr, "b 128");
	cairo_move_to (cr, 5, height-12);
	cairo_show_text (cr, "b -128");
	cairo_move_to (cr, width-75, height/2-3);
	cairo_show_text (cr, "Input 100%");

	ChannelResponse *c=ti3->FirstChannel();
	while(c)
	{
		DeviceNColorant *col=(*ti3)[c->GetChannelIndex()];
		cairo_set_source_rgb (cr, col->red/255.0,col->green/255.0,col->blue/255.0);
//		cerr << "Painting: " << col->GetName() << ": " << col->red << " " << col->green << " " << col->blue << endl;

		int sc=c->GetSampleCount();
		ChannelResponseSample *sam=c->GetSamples();
		cairo_move_to(cr,0,height/2-(paperlab.b*height)/256);
		for(int i=0;i<sc;++i)
		{
			float input=sam[i].input;
			LabValue l(sam[i]);
			input=c->Interpolate(input);

//			cerr << "input " << input << " -> " << l.L << endl;
			cairo_line_to(cr,(input*width)/100.0,height/2-(l.b*height)/256);
			cairo_rel_line_to(cr,2,2);
			cairo_rel_move_to(cr,0,-2);
			cairo_rel_line_to(cr,-2,2);
			cairo_move_to(cr,(input*width)/100.0,height/2-(l.b*height)/256);
		}
		c=c->NextChannel();
		cairo_stroke(cr);
	}
	cairo_destroy (cr);
}


#define DEFAULT_WIDTH 640
#define DEFAULT_HEIGHT 480


int main(int argc, char *argv[])
{
//	stp_init();
	gtk_init(&argc, &argv);
	if(argc>1)
	{
		try
		{
			LinearizationCore core;
			LinProject project(core);
			char *fn;
			if(strlen(argv[1])>5 && strcmp(argv[1]+strlen(argv[1])-4,".ti3")==0)
			{
				fn=strdup(argv[1]);
				project.gprinter.linearization.SetFloat("BaseGamma",1.0);
			}
			else
			{
				project.SetFilename(argv[1]);
				project.Load();
				fn=project.GetTempRoot("RawLin",".ti3");
			}

			double gamma=project.gprinter.linearization.FindFloat("BaseGamma");
			if(strstr(fn,"DropSize.ti3"))
				gamma=project.gprinter.tuning.FindFloat("TuningGamma");;

			RawChannelResponses ti3(fn,gamma);


			free(fn);

			ChannelResponse *c=ti3.FirstChannel();
			int chan=0;
			while(c)
			{
//				c->SetInkLimit(project.gprinter.linearization.InkLimitFromDB(chan));
				++chan;
				c=c->NextChannel();
			}

			Payload payload(ti3,gamma);

			GtkWidget *window;
			GtkWidget *canvas;

			window   = gtk_window_new(GTK_WINDOW_TOPLEVEL);
			g_signal_connect(G_OBJECT (window), "delete-event",
			                G_CALLBACK (gtk_main_quit), NULL);

			GtkWidget *notebook=gtk_notebook_new();
			gtk_container_add(GTK_CONTAINER(window),notebook);
			gtk_widget_show(GTK_WIDGET(notebook));

// Raw Page

			GtkWidget *label=gtk_label_new("Input vs L*,a,b (Raw)");
			gtk_widget_show(label);

			GtkWidget *vbox=gtk_vbox_new(TRUE,0);

			gtk_notebook_append_page(GTK_NOTEBOOK(notebook),GTK_WIDGET(vbox),GTK_WIDGET(label));
			gtk_widget_show(vbox);


			canvas = gtk_drawing_area_new();
			gtk_widget_set_size_request(canvas, DEFAULT_WIDTH, DEFAULT_HEIGHT/2);
			g_signal_connect(G_OBJECT (canvas), "expose-event",
			                G_CALLBACK (paint_input_vs_Lstar_Raw), &payload);
			gtk_box_pack_start(GTK_BOX(vbox),canvas,TRUE,TRUE,0);

			GtkWidget *hbox=gtk_hbox_new(TRUE,0);
			gtk_box_pack_start(GTK_BOX(vbox),hbox,TRUE,TRUE,0);

			canvas = gtk_drawing_area_new();
			gtk_widget_set_size_request(canvas, DEFAULT_WIDTH/2, DEFAULT_HEIGHT/2);
			g_signal_connect(G_OBJECT (canvas), "expose-event",
			                G_CALLBACK (paint_input_vs_a_raw), &payload);
			gtk_box_pack_start(GTK_BOX(hbox),canvas,TRUE,TRUE,0);

			canvas = gtk_drawing_area_new();
			gtk_widget_set_size_request(canvas, DEFAULT_WIDTH/2, DEFAULT_HEIGHT/2);
			g_signal_connect(G_OBJECT (canvas), "expose-event",
			                G_CALLBACK (paint_input_vs_b_raw), &payload);
			gtk_box_pack_start(GTK_BOX(hbox),canvas,TRUE,TRUE,0);


// Gamma Page

			label=gtk_label_new("Input vs L*,a,b (Gamma)");
			gtk_widget_show(label);

			vbox=gtk_vbox_new(TRUE,0);

			gtk_notebook_append_page(GTK_NOTEBOOK(notebook),GTK_WIDGET(vbox),GTK_WIDGET(label));
			gtk_widget_show(vbox);


			canvas = gtk_drawing_area_new();
			gtk_widget_set_size_request(canvas, DEFAULT_WIDTH, DEFAULT_HEIGHT/2);
			g_signal_connect(G_OBJECT (canvas), "expose-event",
			                G_CALLBACK (paint_input_vs_Lstar_Gamma), &ti3);
			gtk_box_pack_start(GTK_BOX(vbox),canvas,TRUE,TRUE,0);

			hbox=gtk_hbox_new(TRUE,0);
			gtk_box_pack_start(GTK_BOX(vbox),hbox,TRUE,TRUE,0);

			canvas = gtk_drawing_area_new();
			gtk_widget_set_size_request(canvas, DEFAULT_WIDTH/2, DEFAULT_HEIGHT/2);
			g_signal_connect(G_OBJECT (canvas), "expose-event",
			                G_CALLBACK (paint_input_vs_a), &ti3);
			gtk_box_pack_start(GTK_BOX(hbox),canvas,TRUE,TRUE,0);

			canvas = gtk_drawing_area_new();
			gtk_widget_set_size_request(canvas, DEFAULT_WIDTH/2, DEFAULT_HEIGHT/2);
			g_signal_connect(G_OBJECT (canvas), "expose-event",
			                G_CALLBACK (paint_input_vs_b), &ti3);
			gtk_box_pack_start(GTK_BOX(hbox),canvas,TRUE,TRUE,0);


// Linearized Page

			label=gtk_label_new("Input vs L*,a,b (linearized)");
			gtk_widget_show(label);

			vbox=gtk_vbox_new(TRUE,0);

			gtk_notebook_append_page(GTK_NOTEBOOK(notebook),GTK_WIDGET(vbox),GTK_WIDGET(label));
			gtk_widget_show(vbox);


			canvas = gtk_drawing_area_new();
			gtk_widget_set_size_request(canvas, DEFAULT_WIDTH, DEFAULT_HEIGHT/2);
			g_signal_connect(G_OBJECT (canvas), "expose-event",
			                G_CALLBACK (paint_input_vs_Lstar_linearized), &ti3);
			gtk_box_pack_start(GTK_BOX(vbox),canvas,TRUE,TRUE,0);

			hbox=gtk_hbox_new(TRUE,0);
			gtk_box_pack_start(GTK_BOX(vbox),hbox,TRUE,TRUE,0);

			canvas = gtk_drawing_area_new();
			gtk_widget_set_size_request(canvas, DEFAULT_WIDTH/2, DEFAULT_HEIGHT/2);
			g_signal_connect(G_OBJECT (canvas), "expose-event",
			                G_CALLBACK (paint_input_vs_a_linearized), &ti3);
			gtk_box_pack_start(GTK_BOX(hbox),canvas,TRUE,TRUE,0);

			canvas = gtk_drawing_area_new();
			gtk_widget_set_size_request(canvas, DEFAULT_WIDTH/2, DEFAULT_HEIGHT/2);
			g_signal_connect(G_OBJECT (canvas), "expose-event",
			                G_CALLBACK (paint_input_vs_b_linearized), &ti3);
			gtk_box_pack_start(GTK_BOX(hbox),canvas,TRUE,TRUE,0);


// a/b plane page

			label=gtk_label_new("a/b plane");
			gtk_widget_show(label);

			vbox=gtk_vbox_new(TRUE,0);

			gtk_notebook_append_page(GTK_NOTEBOOK(notebook),GTK_WIDGET(vbox),GTK_WIDGET(label));
			gtk_widget_show(vbox);


			canvas = gtk_drawing_area_new();
			gtk_widget_set_size_request(canvas, DEFAULT_WIDTH, DEFAULT_HEIGHT);
			g_signal_connect(G_OBJECT (canvas), "expose-event",
			                G_CALLBACK (paint_a_b_plane), &ti3);
			gtk_box_pack_start(GTK_BOX(vbox),canvas,TRUE,TRUE,0);

#if 0
// Input vs C

			label=gtk_label_new("Input vs c (raw)");
			gtk_widget_show(label);

			vbox=gtk_vbox_new(TRUE,0);

			gtk_notebook_append_page(GTK_NOTEBOOK(notebook),GTK_WIDGET(vbox),GTK_WIDGET(label));
			gtk_widget_show(vbox);

			canvas = gtk_drawing_area_new();
			gtk_widget_set_size_request(canvas, DEFAULT_WIDTH, DEFAULT_HEIGHT);
			g_signal_connect(G_OBJECT (canvas), "expose-event",
			                G_CALLBACK (paint_input_vs_c), &ti3);
			gtk_box_pack_start(GTK_BOX(vbox),canvas,TRUE,TRUE,0);
#endif

// Input vs Hue trueness

			label=gtk_label_new("Hue trueness");
			gtk_widget_show(label);

			vbox=gtk_vbox_new(TRUE,0);

			gtk_notebook_append_page(GTK_NOTEBOOK(notebook),GTK_WIDGET(vbox),GTK_WIDGET(label));
			gtk_widget_show(vbox);

			canvas = gtk_drawing_area_new();
			gtk_widget_set_size_request(canvas, DEFAULT_WIDTH, DEFAULT_HEIGHT);
			g_signal_connect(G_OBJECT (canvas), "expose-event",
			                G_CALLBACK (paint_input_vs_huetrueness), &ti3);
			gtk_box_pack_start(GTK_BOX(vbox),canvas,TRUE,TRUE,0);


// Input vs dL by di

			label=gtk_label_new("dL/d_input");
			gtk_widget_show(label);

			vbox=gtk_vbox_new(TRUE,0);

			gtk_notebook_append_page(GTK_NOTEBOOK(notebook),GTK_WIDGET(vbox),GTK_WIDGET(label));
			gtk_widget_show(vbox);

			canvas = gtk_drawing_area_new();
			gtk_widget_set_size_request(canvas, DEFAULT_WIDTH, DEFAULT_HEIGHT);
			g_signal_connect(G_OBJECT (canvas), "expose-event",
			                G_CALLBACK (paint_input_vs_dLbydi), &payload);
			gtk_box_pack_start(GTK_BOX(vbox),canvas,TRUE,TRUE,0);


// Input vs Diminishing Return

			label=gtk_label_new("Dim. Return");
			gtk_widget_show(label);

			vbox=gtk_vbox_new(TRUE,0);

			gtk_notebook_append_page(GTK_NOTEBOOK(notebook),GTK_WIDGET(vbox),GTK_WIDGET(label));
			gtk_widget_show(vbox);

			canvas = gtk_drawing_area_new();
			gtk_widget_set_size_request(canvas, DEFAULT_WIDTH, DEFAULT_HEIGHT);
			g_signal_connect(G_OBJECT (canvas), "expose-event",
			                G_CALLBACK (paint_input_vs_dimreturn), &payload);
			gtk_box_pack_start(GTK_BOX(vbox),canvas,TRUE,TRUE,0);


			gtk_widget_show_all(window);

			gtk_main ();
		}
		catch(const char *err)
		{
			cerr << "Error: " << err << endl;
		}
	}
	return(0);
}


