#include <iostream>

#include <gtk/gtk.h>
#include <gutenprint/gutenprint.h>

#include "config.h"

#include "support/progresstext.h"
#include "progressbar.h"
#include "support/pathsupport.h"
#include "support/util.h"

#include "miscwidgets/generaldialogs.h"

#include "lincore.h"
#include "linproject.h"
#include "projectgui.h"

#include "stepwidget_filename.h"
#include "stepwidget_printsetup.h"
#include "stepwidget_dropsizeinklimit.h"
#include "stepwidget_basedensity.h"
#include "stepwidget_generatedropsizechart.h"
#include "stepwidget_printdropsizechart.h"
#include "stepwidget_dropsizeratios.h"
#include "stepwidget_printcharts.h"
#include "stepwidget_channelinklimits.h"
#include "stepwidget_subchannel.h"

#include "debug.h"

#include "gettext.h"
#define _(x) gettext(x)


using namespace std;


#define DEFAULT_WIDTH 400
#define DEFAULT_HEIGHT 400


int main(int argc,char **argv)
{
//	stp_init();
	gtk_init(&argc,&argv);

	Debug.SetLevel(TRACE);

	const char *filename=NULL;
	if(argc>1)
		filename=argv[1];

	LinearizationCore core;
	ProjectGUI project(core,filename);

	class PrinterDefaults : public PrinterDefaultsCallback
	{
		public:
		PrinterDefaults(LinProject &proj) : PrinterDefaultsCallback(proj) {}
		void SetPrinterDefaults()
		{
			// We default to Raw channels and colour correction,
			// and Drop Size Segmented dither while tuning a printer.
			stp_set_string_parameter(project.gprinter.stpvars,"InputImageType","Raw");
			stp_set_string_parameter(project.gprinter.stpvars,"ColorCorrection","Raw");
			stp_set_string_parameter(project.gprinter.stpvars,"DitherAlgorithm","Segmented");
		}
	} defaultcallback(project);

	project.SetPrinterDefaultsCallback(&defaultcallback);
	project.SetPrinterDefaults();

	project.SetProjectType(LINPROJECTTYPE_TUNING);

	StepSelector_StepList steplist;
	new LinGUIStep_Filename(steplist,_("Choose filename..."),&project);
	new LinGUIStep_PrintSetup(steplist,_("Printer settings..."),&project);
	new LinGUIStep_DropSizeInkLimit(steplist,_("Dropsize limits..."),&project);
//	new LinGUIStep_DropSizeCalibration(steplist,_("Dropsize Calibration..."),&project);
	new LinGUIStep_GenerateDropSizeChart(steplist,_("Generate drop size chart..."),&project);
	new LinGUIStep_PrintDropSizeChart(steplist,_("Print drop size chart..."),&project);
	new LinGUIStep_DropSizeRatios(steplist,_("Drop size ratios..."),&project);
//	new LinGUIStep_ChannelInkLimits(steplist,_("Channel Ink Limits..."),&project);
//	new LinGUIStep_SubChannel(steplist,_("Subchannel Ratios..."),&project);
	project.SetSteps(&steplist);

	if(argc>2)
		project.Import(argv[2]);

	if(!core.CheckBinPath())
		ErrorMessage_Dialog(_("Argyll utilities not found!\nPlease use the Preferences dialog to locate them."));

	gtk_main();
	return(0);
}

