#ifndef IMAGESOURCE_DROPSIZESEGMENT_H
#define IMAGESOURCE_DROPSIZESEGMENT_H

#include "imagesource/imagesource.h"

// Class to take 3 channel input and convert to Drop size segmented mode
// If the first channel is non-zero, it will be mapped to 0x4000 - 0x7fff
// Other wise, if the second channel is non-zero, it will be mapped to
// 0x8000 - 0xbfff.  Finally, if still non-zero, the third channel will
// be mapped to 0xc000 - 0xffff
class ImageSource_DropSizeSegment : public ImageSource
{
	public:
	ImageSource_DropSizeSegment(ImageSource *source) : ImageSource(source), source(source)
	{
		if(source->samplesperpixel<3)
			throw "Dropsize Segmentation needs at least 3 channels";
		samplesperpixel=source->samplesperpixel-2;
		type=IS_TYPE_DEVICEN;
		MakeRowBuffer();
	}
	virtual ~ImageSource_DropSizeSegment()
	{
		if(source)
			delete source;
	}
	virtual ISDataType *GetRow(int row)
	{
		if(row==currentrow)
			return(rowbuffer);
		ISDataType *src=source->GetRow(row);
		for(int x=0;x<width;++x)
		{
			ISDataType t;
			if((t=src[x*source->samplesperpixel]))
				t=(t>>2) + 0x4000;
			else if((t=src[x*source->samplesperpixel+1]))
				t=(t>>2) + 0x8000;
			else if((t=src[x*source->samplesperpixel+2]))
				t=(t>>2) + 0xc000;
			rowbuffer[x*samplesperpixel]=t;
			for(int s=3;s<source->samplesperpixel;++s)
				rowbuffer[x*samplesperpixel+s-2]=(src[x*source->samplesperpixel+s]>>2) + 0xc000;
		}
		currentrow=row;
		return(rowbuffer);
	}	
	protected:
	ImageSource *source;
};

#endif

