#ifndef LINEARIZEDPRINTER_H
#define LINEARIZEDPRINTER_H

// Subclass of GPrinter - applies the linearization to the provided image before printing.

#include "gp_cppsupport/gprinter.h"
#include "imagesource/imagesource_util.h"
#include "linearization.h"
#include "tuning.h"

class ImageSource;
class LinearizedPrinter : public GPrinter
{
	public:
	LinearizedPrinter(PrintOutput &output,ConfigFile *inf,const char *section);
	~LinearizedPrinter();

	// Perform dithering using multiple dropsizes.  Image will be scaled to the printer resolution.
	ImageSource *Dither(ImageSource *source);

	// Perform dithering, mapping chan0 to small black drops, chan1 to medium, chan2 to large
	// then chan3 to Magenta small drops.  Used to print the dropsize calibration target.
	// Image will be scaled to the printer's current resolution.
	ImageSource *DropSizeSeparatedDither(ImageSource *source);

	// Scales a given image to the printer's current resolution - needed if dithering manually.
	// Gutenprint's own scaling can be used if the dithering will be left to Gutenprint.
	ImageSource *ScaleToPrinterResolution(ImageSource *source,IS_ScalingQuality quality=IS_SCALING_NEARESTNEIGHBOUR);

	// Prints an image with linearization applied.  If you don't want the linearization applied
	// use GPrinter::Print instead.
	void Print(ImageSource *source,int xpos,int ypos);

	void PrintPreDithered(ImageSource *source,int xpos,int ypos);

	void SetCurvesFromTI3(const char *fn);
	void SetDropSizesFromTI3(const char *fn);

	Linearization linearization;
	Tuning tuning;
	protected:
};

#endif

