#ifndef LINPROJECT_H
#define LINPROJECT_H

#include "support/configdb.h"
#include "support/searchpath.h"
#include "support/progress.h"
#include "gp_cppsupport/printoutput.h"
#include "gp_cppsupport/gprintersettings.h"
#include "linearizedprinter.h"

#include "argyll_testchartgenerator.h"

#include "lincore.h"

enum LinProjectType {LINPROJECTTYPE_LINEARIZATION,LINPROJECTTYPE_TUNING};

// LinProject provides a base class for project files for the Linearization tools.
// Inherits ConfigFile, and has PrintOutput and GPrinterSettings menbers, which
// between them can track printer queue and driver settings.

class LinProject : public ConfigFile, public ConfigDB
{
	public:
	LinProject(LinearizationCore &core,const char *filename=NULL);
	virtual ~LinProject();
	virtual bool Load();
	virtual bool Save();
	virtual bool Import(const char *filename);
	virtual void SetFilename(const char *filename);
	virtual const char *GetFilename();
	virtual void MakeTempDir();
	virtual const char *GetTempDir();
	virtual char *GetTempRoot(const char *id, const char *ext=NULL);	// Utility function - returns a variation
																		// on the temp base filename, with ID
																		// as a suffix - ie. "/path/to/project_temp/project_ID" 

	// Utility functions to handle getting and setting colorants
	void SetProjectType(enum LinProjectType type=LINPROJECTTYPE_LINEARIZATION);
	void SetColorants(const char *colorants);
	const char *GetColorants();
	int GetStepsPerChannel();
	void SetStepsPerChannel(int steps);

	// Utility function to get and set the Gutenprint InputImageType paramter.
	const char *GetChannelMode();
	void SetChannelMode(const char *mode);

	// High-level chart generation function - gets all the parameters from the project's ConfigDB
	// then uses the Argyll_TestChartGenerator inherited by core to generate the actual chart.
	// Also renders TIFFs from the generated chart.
	virtual void CreateRawChannelLinChart(const char *id="RawLin",Progress *p=NULL);
	virtual TIXFile *GetRawChannelLinChart(const char *id="RawLin");
	virtual Argyll_TestChart_TIFFs *GetRawChannelLinTIFFs(const char *id="RawLin");

	// High-level DropSize Calibration chart function - since the chart we use for this is hand-massaged to output
	// a four-channel file with stepwedges in three channels and spacers in the fourth, we copy it rather than
	// generating it from scratch.
	virtual void CreateDropSizeLinChart(const char *id="DropSize",Progress *p=NULL);
	virtual TIXFile *GetDropSizeLinChart(const char *id="DropSize");
	virtual Argyll_TestChart_TIFFs *GetDropSizeLinTIFFs(const char *id="DropSize");

	// Routine to print an image
	// Automatically handles reinterpreting DeviceN data for CMYK mode (FIXME - and RGB data...)
	virtual void Print(ImageSource *src,int left=0,int top=0,bool bypasslinearization=false);

	PrintOutput printoutput;	// Less confusing to make these members than deal with "diamond inheritance".
	LinearizedPrinter gprinter;
	LinearizationCore &core;
	protected:
	char *filename;
	char *tempdirname;
	char *temproot;
	enum LinProjectType projecttype;
	static ConfigTemplate Template[];
};

#endif

