#include <iostream>
#include <cstring>

using namespace std;

#include "support/util.h"
#include "channelresponse.h"
#include "linearization.h"
#include "linproject.h"
#include "gpdevicensupport.h"
#include "ticgats.h"

#include "imagesource/imagesource_modifiedgamma.h"


double FindSubChannelRatio(double input, ChannelResponse &chan,ChannelResponse &subchan,Linearization &lin)
{
	double result=0.0;

	Interpolate_Lab *li1=chan.GetLabInterp();
	Interpolate_Lab *li2=subchan.GetLabInterp();

	LabValue l2=(*li2)[input];
//	cerr << "L* at " << input << " in " << subchan.GetLongName() << " is " << l2.L << endl;

	double level=li1->ReverseInterpolate_L(l2.L);
//	cerr << "Matching level in " << chan.GetLongName() << " found at " << level << endl;

	// Find raw device values for inklimit in subchannel and corresponding level in main channel
	double corr1=lin.ForwardGamma(input/100.0);
	double corr2=lin.ForwardGamma(level/100.0);
	result=corr2/corr1;

	return(result);
}


int main(int argc,char **argv)
{
//	stp_init();

	if (argc==2)
	{
		try
		{
			LinearizationCore core;
			LinProject proj(core);
			char *fn;
			if(strlen(argv[1])>5 && strcmp(argv[1]+strlen(argv[1])-4,".ti3")==0)
			{
				fn=strdup(argv[1]);
				proj.gprinter.linearization.SetFloat("BaseGamma",0.25);
			}
			else
			{
				proj.SetFilename(argv[1]);
				proj.Load();
				fn=proj.GetTempRoot("RawLin",".ti3");
			}

//			proj.gprinter.SetCurvesFromTI3(fn);

			RawChannelResponses rcr(fn);
			rcr.CalculateInkLimits();
			ChannelResponse *c1=rcr.FirstChannel();
			while(c1)
			{
				ChannelResponse *c2=NULL;
				while((c2=c1->FindSubChannel(c2)))
				{
					cerr << c2->GetLongName() << " is a subchannel of " << c1->GetLongName() << endl;
					cerr << "Ratio: " << FindSubChannelRatio(c1->GetInkLimit(),*c1,*c2,proj.gprinter.linearization) << endl;
					cerr << "Subchannel inklimit: " << c2->GetInkLimit() << endl;
					cerr << "Subchannel inklimit after gamma adjustment: " << proj.gprinter.linearization.ForwardGamma(c2->GetInkLimit()/100.0) * 100.0 << endl;
				}
				c1=c1->NextChannel();
			}

			double based=proj.gprinter.linearization.FindFloat("BaseDensity");
			double baseg=proj.gprinter.linearization.FindFloat("BaseGamma");
			cerr << "Base gamma: " << baseg << endl;
			cerr << "Base density: " << based << endl;
			double fulld=1.0/based;
			cerr << "100% coverage at: " << fulld << endl;

			c1=rcr.FirstChannel();
			while(c1)
			{
				cerr << endl << "Channel: " << c1->GetLongName() << endl;
				Interpolate_Lab *li=c1->GetLabInterp_Smoothed();
				LabValue paper=(*li)[0.0];
				LabValue half=(*li)[50];
				LabValue full=(*li)[100.0];

				// Since we generally never print at 100% density, we estimate an L* value
				// at 100% for use in the dithering algorithms.  To do this we fit an approximate
				// gamma curve to existing testpoints - 50% and 100% linearized, but converted
				// to gutenprint input levels.
				// Our model is this:  y = kx^g, which we solve for k, and g.
				// given two test points, (x1,y1) and (x2,y2), we can derive
				// g = (ln(y2)-ln(y1))/(ln(x2)-ln(x1))
				// k = exp(ln(y1) - g * ln(x1));

				double x1=based*proj.gprinter.linearization.ForwardGamma(0.5)/fulld;
				double x2=based;
				double yr=paper.L-full.L;
				double y1=(paper.L-half.L)/yr;
				double y2=1.0;

				double g=(log(y2)-log(y1))/(log(x2)-log(x1));
				double k=exp(log(y1)-g*log(x1));
				cerr << "x1 : " << x1 << " y1: " << y1 << endl;
				cerr << "x2 : " << x2 << " y2: " << y2 << endl;
				cerr << "Fit gamma: " << g << ", k: " << k << ", L*: " << paper.L-k*yr << endl;

				c1=c1->NextChannel();
			}

			free(fn);
		}
		catch (const char *err)
		{
			cerr << "Error: " << err << endl;
		}
	}
	else
		cerr << "Usage: " << argv[0] << " filename.gplin" << endl;
	return(0);
}

#if 0

int main(int argc,char **argv)
{
	stp_init();

	if (argc==2)
	{
		try
		{
			LinearizationCore core;
			LinProject proj(core,argv[1]);
			proj.Load();

			char *fn=proj.GetTempRoot("RawLin",".ti3");
			proj.gprinter.SetCurvesFromTI3(fn);

			RawChannelResponses rcr(fn);
			rcr.CalculateInkLimits();
			ChannelResponse *c1=rcr.FirstChannel();
			while(c1)
			{
				const char *ln=c1->GetLongName();
				ChannelResponse *c2=rcr.FirstChannel();
				while(c2)
				{
					const char *ln2=c2->GetLongName();

					// Compare channel names - we're looking for one channel being a substring of the other
					if(strstr(ln2,ln) && strcmp(ln2,ln)!=0)
					{
						Interpolate_Lab *li1=c1->GetLabInterp();
						Interpolate_Lab *li2=c2->GetLabInterp();

						LabValue origin=(*li1)[0.0];		// Paper colour

						double ratio;

						// Need to make sure the "subchannel" is indeed lighter
						// than the main channel, to take care of "Dark Yellow", etc.
						if((*li1)[50].L>(*li2)[50].L)
							ratio=FindSubchannelRatio(c1->GetInkLimit(),*c2,*c1,proj.gprinter.linearization);
						else
							ratio=FindSubchannelRatio(c2->GetInkLimit(),*c1,*c2,proj.gprinter.linearization);

						cerr << "Ratio: " << ratio << endl;
					}

					c2=c2->NextChannel();
				}
				c1=c1->NextChannel();
			}

			free(fn);
		}
		catch (const char *err)
		{
			cerr << "Error: " << err << endl;
		}
	}
	else
		cerr << "Usage: " << argv[0] << " filename.gplin" << endl;
	return(0);
}

#endif
#if 0

int main(int argc,char **argv)
{
	stp_init();

	if (argc==2)
	{
		try
		{
			LinearizationCore core;
			LinProject proj(core,argv[1]);
			proj.Load();

			char *fn=proj.GetTempRoot("RawLin",".ti3");
			proj.gprinter.SetCurvesFromTI3(fn);

			RawChannelResponses rcr(fn);
			rcr.CalculateInkLimits();
			ChannelResponse *c1=rcr.FirstChannel();
			while(c1)
			{
				const char *ln=c1->GetLongName();
				ChannelResponse *c2=rcr.FirstChannel();
				while(c2)
				{
					const char *ln2=c2->GetLongName();

					// Compare channel names - we're looking for one channel being a substring of the other
					if(strstr(ln2,ln) && strcmp(ln2,ln)!=0)
					{
						Interpolate_Lab *li1=c1->GetLabInterp();
						Interpolate_Lab *li2=c2->GetLabInterp();

						LabValue origin=(*li1)[0.0];		// Paper colour

						// Need to make sure the "subchannel" is indeed lighter
						// than the main channel, to take care of "Dark Yellow", etc.
						if((*li1)[50].L>(*li2)[50].L)
						{
							Interpolate_Lab *tmp=li1;
							li1=li2;
							li2=tmp;
							// Fixme - need to swap names too!
						}
						cerr << ln2 << " is a subchannel of " << ln << endl;
						double inklimit2=c2->GetInkLimit();

						LabValue l2=(*li2)[inklimit2];
						cerr << "L* at inklimit (" << inklimit2 << ") in " << ln2 << " is " << l2.L << endl;

						double level=li1->ReverseInterpolate_L(l2.L);
						cerr << "Matching level in " << ln << " found at " << level << endl;
						cerr << "L* at " << level << " in " << ln << " is " << (*li1)[level].L << endl;

						// Find raw device values for inklimit in subchannel and corresponding level in main channel
						double corr1=proj.gprinter.linearization.ForwardGamma(inklimit2/100.0);
						double corr2=proj.gprinter.linearization.ForwardGamma(level/100.0);

//						cerr << "Inklimit without correction: " << corr1 << endl;
//						cerr << "Matching level without correction: " << corr2 << endl;
						cerr << "Ratio: " << corr2 / corr1 << endl;

						LabValue l1=(*li1)[level];
						cerr << "dE between channels: " << l1.dE(l2) << endl << endl;

						double bestde=100;
						double besti=0;
						double besti2=0;
						for(double i=inklimit2/2;i<inklimit2;i+=0.1)
						{
							LabValue l2=(*li2)[i];
							double level=li1->ReverseInterpolate_L(l2.L);
							LabValue l1=(*li1)[level];
							double de=l1.dE(l2);

							if(de<bestde)
							{
								bestde=de;
								besti=i;
								besti2=level;
							}
						}
						cerr << "Best match between channels at: " << besti << " (with dE of " << bestde << ")" << endl;
						corr1=proj.gprinter.linearization.ForwardGamma(besti2/100.0);
						corr2=proj.gprinter.linearization.ForwardGamma(besti/100.0);
						cerr << "Subchannel level: " << besti2 << ", ratio:" << corr1/corr2 << endl << endl;

					}

					c2=c2->NextChannel();
				}
				c1=c1->NextChannel();
			}

			free(fn);
		}
		catch (const char *err)
		{
			cerr << "Error: " << err << endl;
		}
	}
	else
		cerr << "Usage: " << argv[0] << " filename.gplin" << endl;
	return(0);
}
#endif

