#include <iostream>

#include "miscwidgets/pixbufview.h"
#include "miscwidgets/generaldialogs.h"
#include "miscwidgets/simplecombo.h"
#include "progressbar.h"

#include "imagesource/imagesource_util.h"
#include "imagesource_devicen_preview.h"
#include "imagesource_devicen_remap.h"
#include "imagesource/pixbuf_from_imagesource.h"

#include "menu_file.h"
#include "menu_edit.h"

#include "stepselector.h"
#include "projectgui.h"
#include "dialogs.h"

#include "config.h"
#include "gettext.h"
#define _(x) gettext(x)
#define N_(x) gettext_noop(x)

using namespace std;

#define DEFAULT_WIDTH 640
#define DEFAULT_HEIGHT 480


static void stepselector_changed(GtkWidget *wid,gpointer userdata)
{
	StepSelector *s=STEPSELECTOR(wid);
	ProjectGUI *gui=(ProjectGUI *)userdata;
	int index=stepselector_get_index(s);
	try
	{
		gui->SetStepWidget(index);
	}
	catch(const char *err)
	{
		ErrorMessage_Dialog(err,gui->window);
	}
}


static void nextbutton_clicked(GtkWidget *wid,gpointer userdata)
{
	ProjectGUI *gui=(ProjectGUI *)userdata;
	try
	{
		gui->NextStep();
	}
	catch(const char *err)
	{
		ErrorMessage_Dialog(err,gui->window);
	}
}


void ProjectGUI::NextStep()
{
	if(stepwidget)
		stepwidget->SaveChanges();
	int index=stepselector_get_index(STEPSELECTOR(stepselector));
	stepselector_set_index(STEPSELECTOR(stepselector),index+1);
}


void ProjectGUI::SetSteps(StepSelector_StepList *steplist)
{
	this->steplist=steplist;
	stepselector_set_steps(STEPSELECTOR(stepselector),steplist);
	stepselector_set_index(STEPSELECTOR(stepselector),0);
	gtk_signal_connect(GTK_OBJECT(stepselector),"changed",(GtkSignalFunc)stepselector_changed,this);
	SetStepWidget(0);
}


void ProjectGUI::SetStepWidget(int idx)
{
	// FIXME - don't want to do this if we've just loaded a new file
	// We can hack around the problem by calling SetStepWidget(0) before loading
	// the new preset.
	if(stepwidget)
	{
		if(stepwidget->IsModified())
		{
			if(Query_Dialog(_("Keep changes?"),window))
				stepwidget->SaveChanges();
			else
				stepwidget->RevertChanges();
		}
		delete stepwidget;
		stepwidget=NULL;
	}
	if(steplist)
	{
		LinGUIStep *step=(LinGUIStep *)(*steplist)[idx];
		if(step)
		{
			if((stepwidget=step->GetWidget()))
			{
				GtkTextBuffer *buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview));
				gtk_text_buffer_set_text(buffer,stepwidget->GetDirections(),-1);

				gtk_box_pack_start(GTK_BOX(vbox),stepwidget->widget,TRUE,TRUE,0);
				gtk_box_reorder_child(GTK_BOX(vbox),stepwidget->widget,2);	// We want this widget between
																			// the two hseparators
				gtk_widget_show(stepwidget->widget);
			}
		}
		int oldidx=stepselector_get_index(STEPSELECTOR(stepselector));
		if(oldidx!=idx)
			stepselector_set_index(STEPSELECTOR(stepselector),idx);

		// Disable the next button if there is no next step...
		gtk_widget_set_sensitive(nextbutton,((*steplist)[idx+1])!=NULL);
	}
}


ProjectGUI::ProjectGUI(LinearizationCore &core,const char *filename)
	: LinProject(core,filename), steplist(NULL), stepwidget(NULL), callback(NULL)
{
	window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW (window), _("Raw Channel Linearization"));
	gtk_window_set_default_size(GTK_WINDOW (window), DEFAULT_WIDTH,DEFAULT_HEIGHT);

	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
		(GtkSignalFunc)gtk_main_quit, NULL);

	GtkWidget *vbox2=gtk_vbox_new(FALSE,0);
	gtk_container_add (GTK_CONTAINER (window), vbox2);

	// Menus

	GtkWidget *menubar;
	GtkAccelGroup *accel_group;
	uimanager = gtk_ui_manager_new();

	BuildFileMenu(this,uimanager);
	BuildEditMenu(&core,uimanager);

	accel_group = gtk_ui_manager_get_accel_group (uimanager);
	gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);

	menubar = gtk_ui_manager_get_widget (uimanager, "/MainMenu");
	gtk_box_pack_start(GTK_BOX(vbox2),menubar,FALSE,TRUE,0);


	// Main window contents

	GtkWidget *hbox=gtk_hbox_new(FALSE,0);
	gtk_box_pack_start(GTK_BOX(vbox2),GTK_WIDGET(hbox),TRUE,TRUE,8);


	// Step selector - vbox mainly for spacing

	vbox=gtk_vbox_new(FALSE,0);
	gtk_box_pack_start(GTK_BOX(hbox),GTK_WIDGET(vbox),FALSE,FALSE,8);

	stepselector=stepselector_new();
	gtk_box_pack_start(GTK_BOX(vbox),GTK_WIDGET(stepselector),TRUE,TRUE,0);


	// Main area

	vbox=gtk_vbox_new(FALSE,0);
	gtk_box_pack_start(GTK_BOX(hbox),GTK_WIDGET(vbox),TRUE,TRUE,8);


	// Directions box

	GtkWidget *sw = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw),GTK_SHADOW_ETCHED_IN);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),GTK_POLICY_NEVER,GTK_POLICY_AUTOMATIC);
	gtk_box_pack_start (GTK_BOX (vbox), sw, FALSE, FALSE, 0);

	textview = gtk_text_view_new ();
	GValue val = { 0, };
	g_value_init (&val, G_TYPE_INT);
	g_value_set_int (&val, 8);
	g_object_set_property(G_OBJECT(textview),"left-margin",&val);
	g_object_set_property(G_OBJECT(textview),"right-margin",&val);
	gtk_text_view_set_editable(GTK_TEXT_VIEW(textview),false);

	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(textview),GTK_WRAP_WORD);

	gtk_container_add(GTK_CONTAINER(sw),textview);


	// Separator

	GtkWidget *tmp=gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(vbox),tmp,FALSE,FALSE,8);


	// Rule off and create "Next" button

	tmp=gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(vbox),tmp,FALSE,FALSE,8);

	GtkWidget *table=gtk_table_new(1,3,TRUE);
	gtk_box_pack_start(GTK_BOX(vbox),table,FALSE,FALSE,0);

	nextbutton=gtk_button_new_with_label(_("Next..."));
	gtk_signal_connect(GTK_OBJECT(nextbutton),"clicked",(GtkSignalFunc)nextbutton_clicked,this);
	gtk_table_attach_defaults(GTK_TABLE(table),nextbutton,2,3,0,1);

	gtk_widget_show_all(window);
}


ProjectGUI::~ProjectGUI()
{
}


GdkPixbuf *ProjectGUI::GetTestChartPreview(TIXFile *ti1,const char *filename,GPColorantList *colorants)
{
	if(!ti1)
		return(NULL);
	GdkPixbuf *result=NULL;
	if(filename)
	{
		int colcount=colorants->GetColorantCount();
		int *channelmap=new int[colcount];
		for(int i=0;i<colcount;++i)
			channelmap[i]=0;

		for(int i=0;i<colorants->GetColorantCount();++i)
		{
			try
			{
				int idx=ti1->GetColorantIndex((*colorants)[i]->GetName());
				if(idx>-1)
					channelmap[idx]=i;
				else
					cerr << "-1: Channel not found" << endl;
			}
			catch(const char *err)
			{
				cerr << "Warning: " << err << endl;
			}
		}

		for(int i=0;i<colcount;++i)
			cerr << "Mapping " << i << " to " << channelmap[i] << endl;

		ImageSource *is=ISLoadImage(filename);

		is=new ImageSource_ToDeviceN(is);
		is=new ImageSource_DeviceN_Remap(is,channelmap,colorants->GetColorantCount());
		ImageSource_DeviceN_Preview dst(is,colorants);
		ProgressBar p("Rendering preview...",false,window,true);  // We make the progress bar modal
		result=pixbuf_from_imagesource(&dst,255,255,255,&p);
		delete[] channelmap;
	}
	return(result);
}


void ProjectGUI::Print(ImageSource *is,int xpos,int ypos,bool bypasslinearization)
{
	ProgressBar p("Printing...",true,window);
	gprinter.SetProgress(&p);
	cerr << "*** ProjectGUI: Printing at position: " << xpos << ", " << ypos << endl;
	LinProject::Print(is,xpos,ypos,bypasslinearization);
	gprinter.SetProgress(NULL);
}


void ProjectGUI::SetPrinterDefaults()
{
	if(callback)
		callback->SetPrinterDefaults();
}


void ProjectGUI::SetPrinterDefaultsCallback(PrinterDefaultsCallback *callback)
{
	this->callback=callback;
}

