#include <iostream>

#include "miscwidgets/generaldialogs.h"
#include "progressbar.h"
#include "support/util.h"
#include "stpui_widgets/stpui_combo.h"
#include "stpui_widgets/dimension.h"

#include "imagesource/imagesource_util.h"
#include "imagesource/imagesource_scaledensity.h"
#include "imagesource/imagesource_devicen_remap.h"

#include "miscwidgets/colorantselector.h"
#include "testpatterns.h"

#include "stepwidget_basedensity.h"

#include "config.h"
#include "gettext.h"
#define _(x) gettext(x)
#define N_(x) gettext_noop(x)


// Base density "step" - self contained class which builds and links all
// the widgets required.  Use the "widget" member to obtain a GtkWidget which can
// be packed into a window


StepWidget_BaseDensity::StepWidget_BaseDensity(ProjectGUI *gui) : StepWidget(gui), colorants(&gui->gprinter)
{
	widget=gtk_vbox_new(FALSE,0);

	GtkWidget *hbox=gtk_hbox_new(FALSE,0);
	gtk_box_pack_start(GTK_BOX(widget),hbox,TRUE,TRUE,0);
	gtk_widget_show(hbox);


	// Colorantselector
	cerr << "Finding Colorants" << endl;
	colorants.SetEnabledColorants(gui->GetColorants());
	cerr << "Done" << endl;
	colorantsel=colorantselector_new(&colorants);
	gtk_box_pack_start(GTK_BOX(hbox),colorantsel,FALSE,FALSE,0);
	gtk_widget_show(colorantsel);
	

	GtkWidget *vbox=gtk_vbox_new(FALSE,0);
	gtk_box_pack_start(GTK_BOX(hbox),vbox,TRUE,TRUE,8);
	gtk_widget_show(vbox);

	GtkWidget *table=gtk_table_new(3,2,FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(table),8);

	gtk_box_pack_start(GTK_BOX(vbox),table,FALSE,FALSE,0);
	gtk_widget_show(table);


	// Papersize selector

	GtkWidget *tmp=gtk_label_new(_("Paper size:"));
//	gtk_box_pack_start(GTK_BOX(vbox),tmp,FALSE,FALSE,0);
	gtk_table_attach_defaults(GTK_TABLE(table),tmp,0,1,0,1);
	gtk_widget_show(tmp);

	papersize=stpui_combo_new(gui->gprinter.stpvars,"PageSize",NULL);
	gtk_table_attach_defaults(GTK_TABLE(table),papersize,1,2,0,1);
//	gtk_box_pack_start(GTK_BOX(vbox),papersize,FALSE,FALSE,4);
	gtk_widget_show(papersize);


	// Top edge selector

	tmp=gtk_label_new(_("Top edge:"));
	gtk_table_attach_defaults(GTK_TABLE(table),tmp,0,1,1,2);
	gtk_widget_show(tmp);

	topedge=dimension_new(0.0,1000.0,UNIT_MILLIMETERS);
	dimension_set_pt(DIMENSION(topedge),14);
	gtk_table_attach_defaults(GTK_TABLE(table),topedge,1,2,1,2);
	gtk_widget_show(topedge);


	// Starting density

	tmp=gtk_label_new(_("Initial density:"));
//	gtk_box_pack_start(GTK_BOX(vbox),tmp,FALSE,FALSE,0);
	gtk_table_attach_defaults(GTK_TABLE(table),tmp,0,1,2,3);
	gtk_widget_show(tmp);

	initialdensity=gtk_spin_button_new_with_range(0,1.0,0.05);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(initialdensity),gui->gprinter.linearization.FindFloat("BaseDensity"));
//	gtk_box_pack_start(GTK_BOX(vbox),initialdensity,FALSE,FALSE,4);
	gtk_table_attach_defaults(GTK_TABLE(table),initialdensity,1,2,2,3);
	g_signal_connect(G_OBJECT(initialdensity),"changed",G_CALLBACK(markmodified),this);
	gtk_widget_show(initialdensity);


	// Print button

	tmp=gtk_button_new_with_label(_("Print Density Evaluation Chart"));
//	gtk_box_pack_start(GTK_BOX(vbox),tmp,FALSE,FALSE,4);
	gtk_table_attach_defaults(GTK_TABLE(table),tmp,0,2,3,4);
	gtk_widget_show(tmp);

	g_signal_connect(G_OBJECT(tmp),"clicked",G_CALLBACK(printclicked),this);


	// Corrected Density

	tmp=gtk_label_new(_("Maximum density\n(read from print):"));
//	gtk_box_pack_start(GTK_BOX(vbox),tmp,FALSE,FALSE,0);
	gtk_table_attach_defaults(GTK_TABLE(table),tmp,0,1,4,5);
	gtk_widget_show(tmp);

	correcteddensity=gtk_spin_button_new_with_range(0,1.0,0.05);
	gtk_table_attach_defaults(GTK_TABLE(table),correcteddensity,1,2,4,5);
//	gtk_box_pack_start(GTK_BOX(vbox),correcteddensity,FALSE,FALSE,4);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(correcteddensity),1.0);
	g_signal_connect(G_OBJECT(correcteddensity),"changed",G_CALLBACK(markmodified),this);
	gtk_widget_show(correcteddensity);


	// Spacer box
	tmp=gtk_vbox_new(FALSE,0);
	gtk_box_pack_start(GTK_BOX(vbox),tmp,TRUE,TRUE,0);
	gtk_widget_show(tmp);
}


// Text for the Directions box

const char *StepWidget_BaseDensity::GetDirections()
{
	return(_("If using \"Raw\" colour correction, please use this page "
		"to determine a suitable base density.\n\n"
		"WARNING!  Improper use of this facility can result in your printer "
		"being flooded with ink!\n\n"
		"Select an initial density then click the Print button.  Read off on "
		"the printed scale the maximum level at which there is no visible paper "
		" soaking or bleeding of ink.\n\n"
		"If there is no bleeding or soaking, increase the initial density and repeat.\n\n"
		"Unless you already know the approximate density, start with a low initial setting "
		"- i.e. 0.5 or so for low resolutions, even lower for high resolutions "
		"- otherwise you run the risk of flooding the printer with ink."));
}


void StepWidget_BaseDensity::SaveChanges()
{
	const char *ps=stp_get_string_parameter(gui->gprinter.stpvars,"PageSize");
	cerr << "Pagesize is currently set to: " << ps << endl;

	float initd=gtk_spin_button_get_value(GTK_SPIN_BUTTON(initialdensity));
	float corrd=gtk_spin_button_get_value(GTK_SPIN_BUTTON(correcteddensity));
	cerr << "Setting density to:" << initd*corrd << endl;
	gui->gprinter.linearization.SetFloat("BaseDensity",initd*corrd);

	char *cols=colorants.GetEnabledColorants();
	cerr << "Selected colorants: " << cols << endl;
	gui->SetColorants(cols);
	free(cols);

	gui->Save();

	modified=false;
}


void StepWidget_BaseDensity::Print()
{
	SaveChanges();
	float initd=gtk_spin_button_get_value(GTK_SPIN_BUTTON(initialdensity));

	// We render the Density Sweep pattern to be a little under 8 inches wide
	ImageSource *is=MakeDensityEvalSweep(colorants,1560,240,2,200);
	ISDeviceNValue density(is->samplesperpixel,ISDataType(initd*IS_SAMPLEMAX));
	ImageSource_ScaleDensity dst(is,density);

	float top=dimension_get_pt(DIMENSION(topedge));

	gui->Print(&dst,20,top,true);	// Bypass linearization since we're applying density manually.
}


// Callback function to mark the step as modified...
void StepWidget_BaseDensity::markmodified(GtkWidget *wid,gpointer data)
{
	StepWidget_BaseDensity *step=(StepWidget_BaseDensity *)data;
	step->modified=true;
}


// Callback function for the Print button
void StepWidget_BaseDensity::printclicked(GtkWidget *wid,gpointer data)
{
	StepWidget_BaseDensity *step=(StepWidget_BaseDensity *)data;
	try
	{
		step->Print();
	}
	catch(const char *err)
	{
		ErrorMessage_Dialog(err,step->gui->window);
	}
}

