#include <iostream>

#include <cairo.h>

#include "miscwidgets/generaldialogs.h"
#include "progressbar.h"
#include "support/util.h"
#include "stpui_widgets/stpui_combo.h"
#include "stpui_widgets/dimension.h"

#include "imagesource/imagesource_util.h"
#include "imagesource/imagesource_scaledensity.h"
#include "imagesource/imagesource_modifiedgamma.h"
#include "imagesource/imagesource_gamma.h"
#include "imagesource/imagesource_devicen_remap.h"

#include "testpatterns.h"
#include "miscwidgets/colorantselector.h"

#include "stepwidget_channelinklimits.h"

#include "config.h"
#include "gettext.h"
#define _(x) gettext(x)
#define N_(x) gettext_noop(x)


// Class which encapsulates the creation and operation of an ink limit widget
// The StepWidget will contain an array of these, one for each channel.

class InkLimit_Payload
{
	public:
	InkLimit_Payload(StepWidget_ChannelInkLimits &step,int index)
		: label(NULL), widget(NULL), step(step)
	{
		channel=step.chans->GetChannel(index);

		cerr << "Creating widget for " << channel->GetShortName() << endl;
		label=gtk_label_new(channel->GetShortName());
		gtk_widget_show(label);

		widget=gtk_spin_button_new_with_range(0,100,1);
		Refresh();
		g_signal_connect(G_OBJECT(widget),"value-changed",G_CALLBACK(limitchanged),this);
		gtk_widget_show(widget);
	}
	~InkLimit_Payload()
	{
	}
	void Refresh()
	{
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget),channel->GetInkLimit());
	}
	double GetLimit()
	{
		return(gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget)));
	}
	static void limitchanged(GtkWidget *wid,gpointer data)
	{
		InkLimit_Payload *p=(InkLimit_Payload *)data;
		StepWidget_ChannelInkLimits::markmodified(wid,&p->step);
		double limit=gtk_spin_button_get_value(GTK_SPIN_BUTTON(p->widget));
		p->channel->SetInkLimit(limit);
		gtk_widget_queue_draw (GTK_WIDGET (p->step.notebook));
	}
	GtkWidget *label;
	GtkWidget *widget;
	protected:
	ChannelResponse *channel;
	StepWidget_ChannelInkLimits &step;
};



// Painting functions for graphs


// Hue trueness


static void paint_hue_trueness (GtkWidget *widget,GdkEventExpose *eev,gpointer userdata)
{
	RawChannelResponses *ti3=(RawChannelResponses *)userdata;
	if(!ti3)
		return;

	int width  = widget->allocation.width;
	int height = widget->allocation.height;

	cairo_t *cr = gdk_cairo_create (widget->window);
 
	cairo_set_source_rgb (cr, 1,1,1);
	cairo_paint (cr);

	cairo_set_source_rgb (cr, 0,0,0);
	cairo_set_line_width(cr,1.0);
	cairo_move_to (cr,0,0);
	cairo_line_to(cr,0,height-1);
	cairo_line_to(cr,width-1,height-1);
	cairo_stroke (cr);

    cairo_select_font_face (cr, "Sans", CAIRO_FONT_SLANT_ITALIC,
                                        CAIRO_FONT_WEIGHT_BOLD);

	cairo_set_font_size (cr, 15);
	cairo_move_to (cr, width-60, height-5);
	cairo_show_text (cr, "Input");

	ChannelResponse *c=ti3->FirstChannel();
	while(c)
	{
		DeviceNColorant *col=(*ti3)[c->GetChannelIndex()];
		cairo_set_source_rgb (cr, col->red/255.0,col->green/255.0,col->blue/255.0);
//		cerr << "Painting: " << col->GetName() << ": " << col->red << " " << col->green << " " << col->blue << endl;

		int sc=c->GetSampleCount();
		ChannelResponseSample *sam=c->GetSamples();
		Interpolate_Lab *labint=c->GetLabInterp_Smoothed();
		LabValue origin(sam[0]);
		cairo_move_to(cr,0,1);
		float slope=1.0;
		for(int i=2;i<sc-2;++i)
		{
			LabValue prev=(*labint)[sam[i-1].input];
			LabValue current=(*labint)[sam[i+1].input];
//			LabValue prev(sam[i-2]);
//			LabValue current(sam[i+2]);

			float d1=(prev-origin).Magnitude();
			float d2=(current-origin).Magnitude();
			float d3=(current-prev).Magnitude();

			slope=(d2-d1)/d3;

			cairo_line_to(cr,(i*width)/sc,height-(slope*height)-1);
			cairo_rel_line_to(cr,2,2);
			cairo_rel_move_to(cr,0,-2);
			cairo_rel_line_to(cr,-2,2);
			cairo_line_to(cr,(i*width)/sc,height-(slope*height)-1);
		}
		c=c->NextChannel();
		cairo_stroke(cr);
	}
	cairo_destroy (cr);
}


// a/b plane


static void paint_a_b_plane (GtkWidget *widget,GdkEventExpose *eev,gpointer userdata)
{
	RawChannelResponses *ti3=(RawChannelResponses *)userdata;
	if(!ti3)
		return;

	int width  = widget->allocation.width;
	int height = widget->allocation.height;

	cairo_t *cr = gdk_cairo_create (widget->window);
 
	cairo_set_source_rgb (cr, 1,1,1);
	cairo_paint (cr);

	cairo_set_source_rgb (cr, 0,0,0);
	cairo_set_line_width(cr,1.0);
	cairo_move_to (cr, width/2,0);
	cairo_line_to(cr,width/2,height);
	cairo_move_to (cr, 0, height/2);
	cairo_line_to(cr,width,height/2);
	cairo_stroke (cr);

    cairo_select_font_face (cr, "Sans", CAIRO_FONT_SLANT_ITALIC,
                                        CAIRO_FONT_WEIGHT_BOLD);
	cairo_set_font_size (cr, 9);
	cairo_move_to (cr, width/2+5, 12);
	cairo_show_text (cr, "+128");
	cairo_move_to (cr, width/2+5, height-3);
	cairo_show_text (cr, "-128");

	cairo_move_to (cr, 5, height/2+15);
	cairo_show_text (cr, "-128");
	cairo_move_to (cr, width-30, height/2+15);
	cairo_show_text (cr, "+128");

	cairo_set_font_size (cr, 15);
	cairo_move_to (cr, width/2+50, 20);
	cairo_show_text (cr, "b");
	cairo_move_to (cr, width-20, height/2+35);
	cairo_show_text (cr, "a");

	ChannelResponse *c=ti3->FirstChannel();
	while(c)
	{
		DeviceNColorant *col=(*ti3)[c->GetChannelIndex()];
		cairo_set_source_rgb (cr, col->red/255.0,col->green/255.0,col->blue/255.0);

		double inklimit=c->GetInkLimit();
		bool greyed=false;

		int sc=c->GetSampleCount();
		ChannelResponseSample *sam=c->GetSamples();
		for(int i=0;i<sc;++i)
		{
			if(sam[i].input>inklimit && !greyed)
			{
				cairo_stroke(cr);
				int g=(col->red+col->green+col->blue);
//				cairo_set_source_rgb (cr, (col->red+g)/511.0,(col->green+g)/511.0,(col->blue+g)/511.0);
				cairo_set_source_rgb (cr, g/765.0,g/765.0,g/765.0);
				greyed=true;
			}
			LabValue l(sam[i]);
			cairo_move_to(cr,width/2+(l.a*width)/256-1,height/2-(l.b*height)/256-1);
			cairo_rel_line_to(cr,2,2);
			cairo_rel_move_to(cr,0,-2);
			cairo_rel_line_to(cr,-2,2);
			cairo_line_to(cr,width/2+(l.a*width)/256-1,height/2-(l.b*height)/256-1);
		}
		c=c->NextChannel();
		cairo_stroke(cr);
	}
	cairo_destroy (cr);
}


static void paint_input_vs_Lstar_linearized (GtkWidget *widget,GdkEventExpose *eev,gpointer userdata)
{
	RawChannelResponses *ti3=(RawChannelResponses *)userdata;
	if(!ti3)
		return;
	ChannelResponseSample *paper=ti3->GetPaper();
	LabValue paperlab(*paper);

	int width  = widget->allocation.width;
	int height = widget->allocation.height;

	cairo_t *cr = gdk_cairo_create (widget->window);
 
	cairo_set_source_rgb (cr, 1,1,1);
	cairo_paint (cr);

	cairo_set_source_rgb (cr, 0,0,0);
	cairo_set_line_width(cr,1.0);
	cairo_move_to (cr, 1,0);
	cairo_line_to(cr,1,height);
	cairo_move_to(cr,0,height-1);
	cairo_line_to (cr, width,height-1);
	cairo_stroke (cr);

    cairo_select_font_face (cr, "Sans", CAIRO_FONT_SLANT_ITALIC,
                                        CAIRO_FONT_WEIGHT_BOLD);
	cairo_set_font_size (cr, 9);
	cairo_move_to (cr, 5, 12);
	cairo_show_text (cr, "L* 100");
	cairo_move_to (cr, width-75, height-3);
	cairo_show_text (cr, "Input 100%");

	ChannelResponse *c=ti3->FirstChannel();
	while(c)
	{
		DeviceNColorant *col=(*ti3)[c->GetChannelIndex()];
		cairo_set_source_rgb (cr, col->red/255.0,col->green/255.0,col->blue/255.0);

		int sc=c->GetSampleCount();
		ChannelResponseSample *sam=c->GetSamples();

		double inklimit=c->GetInkLimit();
		Interpolate_Lab *labinterp=c->GetLabInterp();
		LabValue lmax=(*labinterp)[0.0];
		LabValue lmin=(*labinterp)[inklimit];

		cairo_move_to(cr,0,height-(paperlab.L*height)/100);
		for(int i=0;i<sc;++i)
		{
			float input=sam[i].input;
			if(input<=inklimit)
			{
				LabValue l(sam[i]);
				if(c->GetType()==CHANNELRESPONSETYPE_YELLOW)
				{
					input=c->Interpolate(input);
				}
				else
				{
					input=c->Interpolate(input);
				}
				cairo_move_to(cr,(input*width)/100.0,height-(l.L*height)/100.0);
				cairo_rel_line_to(cr,2,2);
				cairo_rel_move_to(cr,0,-2);
				cairo_rel_line_to(cr,-2,2);
				cairo_line_to(cr,(input*width)/100.0,height-(l.L*height)/100.0);
			}
		}

		cairo_move_to(cr,0,height-(height*lmax.L)/100.0);
		cairo_line_to(cr,width-1,height-(height*lmin.L)/100.0);

		c=c->NextChannel();
		cairo_stroke(cr);
	}
	cairo_destroy (cr);
}


// Callback function to calculate default ink limits
void StepWidget_ChannelInkLimits::defaultinklimits(GtkWidget *wid,gpointer data)
{
	StepWidget_ChannelInkLimits *step=(StepWidget_ChannelInkLimits *)data;
	if(step->chans)
	{
		step->chans->CalculateInkLimits();

		int idx=0;
		ChannelResponse *c=step->chans->FirstChannel();
		while(c)
		{
			step->channel_payloads[idx]->Refresh();
			++idx;
			c=c->NextChannel();
		}
	}
}



// Individual channel ink limit "step" - self contained class which builds and links all
// the widgets required.  Use the "widget" member to obtain a GtkWidget which can
// be packed into a window


StepWidget_ChannelInkLimits::StepWidget_ChannelInkLimits(ProjectGUI *gui) : StepWidget(gui), chans(NULL)
{
	for(int i=0;i<LINEARIZATION_MAX_CHANNELS;++i)
		channel_payloads[i]=NULL;

	char *fn=gui->GetTempRoot("RawLin",".ti3");
	try
	{
		chans=new RawChannelResponses(fn,gui->gprinter.linearization.FindFloat("BaseGamma"));
	}
	catch (const char *err)
	{
		cerr << "Couldn't get ChannelResponses - chart probably hasn't been read yet!" << endl;
	}

	free(fn);

	widget=gtk_vbox_new(FALSE,0);

	GtkWidget *hbox=gtk_hbox_new(FALSE,8);
	gtk_widget_show(hbox);

	gtk_box_pack_start(GTK_BOX(widget),hbox,TRUE,TRUE,0);

	notebook=gtk_notebook_new();
	gtk_box_pack_start(GTK_BOX(hbox),notebook,TRUE,TRUE,0);
	gtk_widget_show(GTK_WIDGET(notebook));



//	Ink limits page
	
	GtkWidget *vbox=gtk_vbox_new(FALSE,0);
	gtk_widget_show(vbox);
	gtk_box_pack_start(GTK_BOX(hbox),vbox,FALSE,FALSE,0);

	if(chans)
	{
		GtkWidget *table=gtk_table_new(1,1,FALSE);
		gtk_table_set_col_spacings(GTK_TABLE(table),4);
		gtk_table_set_row_spacings(GTK_TABLE(table),12);
		gtk_box_pack_start(GTK_BOX(vbox),table,FALSE,FALSE,8);
		gtk_widget_show(table);

		int idx=0;

		ChannelResponse *c=chans->FirstChannel();
		while(c)
		{
			// Set a default ink limit - we use the limit from the DB
			// unless it's at override it.
			double limit=gui->gprinter.linearization.InkLimitFromDB(idx);
			if(limit==100.0)
				c->CalculateInkLimit();
			else
				c->SetInkLimit(limit);

			// Build self-contained widget for adjusting ink limit
			channel_payloads[idx]=new InkLimit_Payload(*this,idx);

			int row=idx/2;
			int col=2*(idx&1);

			gtk_table_attach_defaults(GTK_TABLE(table),channel_payloads[idx]->label,col,col+1,row,row+1);
			gtk_table_attach_defaults(GTK_TABLE(table),channel_payloads[idx]->widget,col+1,col+2,row,row+1);

			++idx;
			c=c->NextChannel();
		}
		if(idx&1) ++idx;

		GtkWidget *button=gtk_button_new_with_label(_("Defaults"));
		gtk_widget_show(button);
		g_signal_connect(G_OBJECT (button), "clicked", G_CALLBACK (defaultinklimits), this);
		gtk_table_attach_defaults(GTK_TABLE(table),button,1,4,idx/2+1,idx/2+2);
	}

//	Linearized channel response graph

	GtkWidget *label=gtk_label_new("Input vs L* (linearized)");
	gtk_widget_show(label);


	GtkWidget *canvas = gtk_drawing_area_new();
	gtk_widget_set_size_request(canvas, 50,50);
	g_signal_connect(G_OBJECT (canvas), "expose-event",
	                G_CALLBACK (paint_input_vs_Lstar_linearized), chans);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),GTK_WIDGET(canvas),GTK_WIDGET(label));
	gtk_widget_show(canvas);

// a/b plane page

	label=gtk_label_new("a/b plane");
	gtk_widget_show(label);

	canvas = gtk_drawing_area_new();
	gtk_widget_set_size_request(canvas, 50, 50);
	g_signal_connect(G_OBJECT (canvas), "expose-event",
	                G_CALLBACK (paint_a_b_plane), chans);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),GTK_WIDGET(canvas),GTK_WIDGET(label));
	gtk_widget_show(canvas);


// Hue trueness page

	label=gtk_label_new("Hue trueness");
	gtk_widget_show(label);

	canvas = gtk_drawing_area_new();
	gtk_widget_set_size_request(canvas, 50, 50);
	g_signal_connect(G_OBJECT (canvas), "expose-event",
	                G_CALLBACK (paint_hue_trueness), chans);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),GTK_WIDGET(canvas),GTK_WIDGET(label));
	gtk_widget_show(canvas);


	if(!chans)
		gtk_widget_set_sensitive(GTK_WIDGET(notebook),FALSE);

}


StepWidget_ChannelInkLimits::~StepWidget_ChannelInkLimits()
{
	for(int i=0;i<LINEARIZATION_MAX_CHANNELS;++i)
	{
		if(channel_payloads[i])
			delete(channel_payloads[i]);
	}
	if(chans)
		delete chans;
}


// Text for the Directions box

const char *StepWidget_ChannelInkLimits::GetDirections()
{
	if(chans)
	{
		return(_("This page is used to set ink limits for the individual channels.\n"
				 "Suitable Ink limits will be calculated automatically, but you can override these if you wish"));
	}
	else
	{
		return(_("Unable to open the .ti3 file, which will be created when reading the linearization chart.\n\n"
				 "One this chart has been read, this page can be used to set ink limits for the individual channels.\n"
				 "Suitable Ink limits will be calculated automatically, but you can override these if you wish"));
	}
}


void StepWidget_ChannelInkLimits::SaveChanges()
{
	if(chans)
	{
		int i=0;
		ChannelResponse *c=chans->FirstChannel();
		while(c)
		{
			if(channel_payloads[i])
			{
				double limit=channel_payloads[i]->GetLimit();
				gui->gprinter.linearization.InkLimitToDB(i,limit);
			}
			++i;
			c=c->NextChannel();
		}
	}
		
	gui->Save();

	modified=false;
}


// Callback function to mark the step as modified...
void StepWidget_ChannelInkLimits::markmodified(GtkWidget *wid,gpointer data)
{
	StepWidget_ChannelInkLimits *step=(StepWidget_ChannelInkLimits *)data;
	step->modified=true;
}


