#include <iostream>

#include <gtk/gtk.h>

#include "miscwidgets/generaldialogs.h"
#include "miscwidgets/pixbufview.h"
#include "miscwidgets/imageselector.h"
#include "support/util.h"
#include "progressbar.h"

#include "stpui_widgets/stpui_combo.h"
#include "stpui_widgets/stpui_printerselector.h"
#include "stpui_widgets/stpui_optionbook.h"
#include "gp_cppsupport/printoutputselector.h"

#include "imagesource/imagesource_util.h"
#include "imagesource/imagesource_scaledensity.h"
#include "imagesource/imagesource_modifiedgamma.h"
#include "imagesource/imagesource_devicen_remap.h"
#include "imagesource_dropsizesegment.h"

#include "stepwidget_printdropsizechart.h"

#include "debug.h"

#include "config.h"
#include "gettext.h"
#define _(x) gettext(x)
#define N_(x) gettext_noop(x)


#define MANUAL_DITHER

// Populating the image list is deferred until after the widget has been created
// and displayed
// FIXME - this still doesn't solve the race condition of the user
// selecting a different page while this process is running...
// Maybe make the progress indicator modal?

gboolean StepWidget_PrintDropSizeChart::populate(gpointer ud)
{
	StepWidget_PrintDropSizeChart *step=(StepWidget_PrintDropSizeChart *)ud;

	if(step->tiffs)
	{
		char *fn=step->tiffs->GetFirstFilename();
		while(fn)
		{
			char *jpegfn=BuildFilename(fn,"","jpg");
			if(g_path_is_absolute(jpegfn))
			{
				imageselector_set_filename(IMAGESELECTOR(step->imageselector),jpegfn);
			}
			else
			{
				char *absfn=BuildAbsoluteFilename(jpegfn);
				imageselector_set_filename(IMAGESELECTOR(step->imageselector),absfn);
				free(absfn);
			}
			free(jpegfn);
			free(fn);
			fn=step->tiffs->GetNextFilename();
		}
	}

	g_signal_connect(G_OBJECT(step->imageselector),"changed",G_CALLBACK(imagechanged),step);

	step->DisplayChart();

	return(FALSE);
}


// Chart printing "step"

StepWidget_PrintDropSizeChart::StepWidget_PrintDropSizeChart(ProjectGUI *gui) : StepWidget(gui), colorants(&gui->gprinter), ti1(NULL), tiffs(NULL)
{
	try
	{
		ti1=gui->GetDropSizeLinChart("DropSize");
		char *fn=gui->GetTempRoot("DropSize");
		if(fn)
		{
			tiffs=new Argyll_TestChart_TIFFs(fn);
			free(fn);
		}
	}
	catch(const char *err)
	{
		cerr << "Error: " << err << endl;
	}

	widget=gtk_vbox_new(FALSE,0);

	GtkWidget *hbox=gtk_hbox_new(FALSE,0);
	gtk_box_pack_start(GTK_BOX(widget),GTK_WIDGET(hbox),TRUE,TRUE,0);
	gtk_widget_show(hbox);

	GtkWidget *vbox=gtk_vbox_new(FALSE,0);
	gtk_box_pack_start(GTK_BOX(hbox),GTK_WIDGET(vbox),TRUE,TRUE,0);
	gtk_widget_show(vbox);

	// PixbufView

	pview=pixbufview_new(NULL,false);
	gtk_box_pack_start(GTK_BOX(vbox),GTK_WIDGET(pview),TRUE,TRUE,0);
	gtk_widget_show(pview);


	// Print Button

	GtkWidget *tmp=gtk_button_new_with_label(_("Print test chart"));
	gtk_box_pack_start(GTK_BOX(vbox),GTK_WIDGET(tmp),FALSE,FALSE,0);
	g_signal_connect(G_OBJECT(tmp),"clicked",G_CALLBACK(printclicked),this);
	gtk_widget_show(tmp);


	// ImageSelector

	imageselector=imageselector_new(NULL,GTK_SELECTION_SINGLE,false);
	gtk_box_pack_start(GTK_BOX(hbox),GTK_WIDGET(imageselector),FALSE,FALSE,8);
	gtk_widget_show(imageselector);

	g_timeout_add(1,populate,this);
}


StepWidget_PrintDropSizeChart::~StepWidget_PrintDropSizeChart()
{
	if(tiffs)
		delete tiffs;
	if(ti1)
		delete ti1;
}


const char *StepWidget_PrintDropSizeChart::GetDirections()
{
	return(_("Test-chart printing:  Please use the image selector on the right to "
			"select a page.  You can examine the page in the preview below. "
			"Use the Print button to print each page in turn."));
}


void StepWidget_PrintDropSizeChart::DisplayChart()
{
	const char *jfn=imageselector_get_filename(IMAGESELECTOR(imageselector));
	if(jfn)
	{
		char *fn=BuildFilename(jfn,"","tif");
		if(fn)
		{
			GdkPixbuf *pb=gui->GetTestChartPreview(ti1,fn,&colorants);
			if(pb)
			{
				pixbufview_set_pixbuf(PIXBUFVIEW(pview),pb);
				g_object_unref(G_OBJECT(pb));
			}
			free(fn);
		}
	}
}


void StepWidget_PrintDropSizeChart::PrintChart()
{
	// FIXME - here we have to do the double-remap between TI1 and DropSize_Colorants,
	// then to the printer's real colorants.
	Debug[TRACE] << "In PrintChart()" << endl;
	const char *jfn=imageselector_get_filename(IMAGESELECTOR(imageselector));
	if(jfn)
	{
		char *fn=BuildFilename(jfn,"","tif");
		if(fn)
		{
			try
			{
				int colcount=colorants.GetColorantCount();
				int *channelmap=ti1->CreateRawChannelMap(colorants);

				ImageSource *is=ISLoadImage(fn);

				is=new ImageSource_ToDeviceN(is);
				if(channelmap)
				{
					cerr << "Have a channel map" << endl;
					is=new ImageSource_DeviceN_Remap(is,channelmap,colcount);
					delete[] channelmap;
				}
				else
					cerr << "No channel map" << endl;

#ifdef MANUAL_DITHER
				is=gui->gprinter.DropSizeSeparatedDither(is);
				gui->gprinter.PrintPreDithered(is,0,0);
#else
				ISDeviceNValue densities(is->samplesperpixel,IS_SAMPLEMAX);
				densities[0]=IS_SAMPLEMAX*gui->gprinter.tuning.FindFloat("DropSizeLimit0");
				densities[1]=IS_SAMPLEMAX*gui->gprinter.tuning.FindFloat("DropSizeLimit1");
				densities[2]=IS_SAMPLEMAX*gui->gprinter.tuning.FindFloat("DropSizeLimit2");
				densities[3]=IS_SAMPLEMAX*gui->gprinter.tuning.FindFloat("DropSizeLimit2");
				// We print Magenta at the same density as the largest black dropsize.

				// Apply rough gamma correction, then density scaling
				is=new ImageSource_ModifiedGamma(is,gui->gprinter.tuning.FindFloat("TuningGamma"));
				is=new ImageSource_ScaleDensity(is,densities);

				// Now convert to drop-size-segnmented...
				is=new ImageSource_DropSizeSegment(is);

				Debug[TRACE] << "Remapping to DeviceN" << endl;

				// Remap to printer DeviceN
				GPColorantList gpcolorants(&gui->gprinter);
				colcount=gpcolorants.GetColorantCount();
				int blackidx=gpcolorants.GetColorantIndex("Black");
				if(blackidx<0)
					blackidx=gpcolorants.GetColorantIndex("Photo Black");
				cerr << "Got black index:" << blackidx << endl;
				int maptable[2]={0,0};
				maptable[0]=blackidx;
				int magidx=gpcolorants.GetColorantIndex("Magenta");
				if(magidx<0)
					magidx=gpcolorants.GetColorantIndex("Vivid Magenta");
				cerr << "Got magenta index:" << magidx << endl;
				maptable[1]=magidx;

				ImageSource_DeviceN_Remap dst(is,maptable,colcount);

				Debug[TRACE] << "About to print" << endl;

				gui->Print(&dst,0,0,true);	// Bypass linearization since we're applying density manually.
#endif
			}
			catch (const char *err)
			{
				ErrorMessage_Dialog(err,gui->window);
			}
			free(fn);
		}
	}
}


void StepWidget_PrintDropSizeChart::printclicked(GtkWidget *wid,gpointer data)
{
	StepWidget_PrintDropSizeChart *step=(StepWidget_PrintDropSizeChart *)data;
	try
	{
		step->PrintChart();
	}
	catch(const char *err)
	{
		ErrorMessage_Dialog(err,step->gui->window);
	}
}


void StepWidget_PrintDropSizeChart::imagechanged(GtkWidget *wid,gpointer data)
{
	StepWidget_PrintDropSizeChart *step=(StepWidget_PrintDropSizeChart *)data;
	try
	{
		step->DisplayChart();
	}
	catch(const char *err)
	{
		ErrorMessage_Dialog(err,step->gui->window);
	}
}

