#include <iostream>

#include <cairo.h>

#include "miscwidgets/generaldialogs.h"
#include "progressbar.h"
#include "support/util.h"
#include "stpui_widgets/stpui_combo.h"
#include "stpui_widgets/dimension.h"

#include "imagesource/imagesource_util.h"
#include "imagesource/imagesource_scaledensity.h"
#include "imagesource/imagesource_modifiedgamma.h"
#include "imagesource/imagesource_gamma.h"
#include "imagesource/imagesource_devicen_remap.h"

#include "testpatterns.h"
#include "miscwidgets/colorantselector.h"

#include "stepwidget_subchannel.h"

#include "config.h"
#include "gettext.h"
#define _(x) gettext(x)
#define N_(x) gettext_noop(x)


// Class containing data needed by the graph-drawing callback functions

class SubChannel_Payload
{
	public:
	SubChannel_Payload(StepWidget_SubChannel &step) : step(step)
	{
		ti3=step.chans;
	}
	~SubChannel_Payload()
	{
	}
	double FindSubChannelRatio(double input, ChannelResponse &chan,ChannelResponse &subchan,Linearization &lin)
	{
		double result=0.0;

		Interpolate_Lab *li1=chan.GetLabInterp();
		Interpolate_Lab *li2=subchan.GetLabInterp();

		LabValue l2=(*li2)[input];
	//	cerr << "L* at " << input << " in " << subchan.GetLongName() << " is " << l2.L << endl;

		double level=li1->ReverseInterpolate_L(l2.L);
	//	cerr << "Matching level in " << chan.GetLongName() << " found at " << level << endl;

		// Find raw device values for inklimit in subchannel and corresponding level in main channel
		double corr1=lin.ForwardGamma(input/100.0);
		double corr2=lin.ForwardGamma(level/100.0);
		result=corr2/corr1;

		return(result);
	}
	StepWidget_SubChannel &step;
	RawChannelResponses *ti3;
};



// Graph drawing functions

// Input vs. SubChannel Ratio

void StepWidget_SubChannel::paint_input_vs_subchannel_ratio(GtkWidget *widget,GdkEventExpose *eev,gpointer userdata)
{
	SubChannel_Payload *payload=(SubChannel_Payload *)userdata;
	RawChannelResponses *ti3=payload->ti3;

	int width  = widget->allocation.width;
	int height = widget->allocation.height;

	cairo_t *cr = gdk_cairo_create (widget->window);
 
	cairo_set_source_rgb (cr, 1,1,1);
	cairo_paint (cr);

	cairo_set_source_rgb (cr, 0,0,0);
	cairo_set_line_width(cr,1.0);
	cairo_move_to (cr, 1,0);
	cairo_line_to(cr,1,height);
	cairo_move_to(cr,0,height-1);
	cairo_line_to (cr, width,height-1);
	cairo_stroke (cr);

    cairo_select_font_face (cr, "Sans", CAIRO_FONT_SLANT_ITALIC,
                                        CAIRO_FONT_WEIGHT_BOLD);
	cairo_set_font_size (cr, 9);
	cairo_move_to (cr, width-75, height-3);
	cairo_show_text (cr, "Input 100%");

	ChannelResponse *c=ti3->FirstChannel();
	while(c)
	{
		ChannelResponse *c2=NULL;
		while((c2=c->FindSubChannel(c2)))
		{
			cerr << c2->GetLongName() << " is a subchannel of " << c->GetLongName() << endl;
			DeviceNColorant *col=(*ti3)[c->GetChannelIndex()];
			cairo_set_source_rgb (cr, col->red/255.0,col->green/255.0,col->blue/255.0);

			bool first=true;

			for(int i=1;i<100;++i)
			{
				float input=i;
				double ratio=payload->FindSubChannelRatio(input,*c,*c2,payload->step.gui->gprinter.linearization);

				if(first)
					cairo_move_to(cr,(input*width)/100.0,height-(ratio*height));
				else
					cairo_line_to(cr,(input*width)/100.0,height-(ratio*height));
				cairo_rel_line_to(cr,2,2);
				cairo_rel_move_to(cr,0,-2);
				cairo_rel_line_to(cr,-2,2);
				cairo_move_to(cr,(input*width)/100.0,height-(ratio*height));
				first=false;
			}
			cairo_stroke(cr);
		}
		c=c->NextChannel();
	}
	cairo_destroy (cr);
}


// Input vs. dL/d_input
// Only paints channels which are subchannels of others.


void StepWidget_SubChannel::paint_input_vs_dLbydi (GtkWidget *widget,GdkEventExpose *eev,gpointer userdata)
{
	SubChannel_Payload *payload=(SubChannel_Payload *)userdata;
	RawChannelResponses *ti3=payload->ti3;
	Linearization &lin=payload->step.gui->gprinter.linearization;

	int width  = widget->allocation.width;
	int height = widget->allocation.height;

	cairo_t *cr = gdk_cairo_create (widget->window);
 
	cairo_set_source_rgb (cr, 1,1,1);
	cairo_paint (cr);

	cairo_set_source_rgb (cr, 0,0,0);
	cairo_set_line_width(cr,1.0);
	cairo_move_to (cr,0,0);
	cairo_line_to(cr,0,height-1);
	cairo_line_to(cr,width-1,height-1);
	cairo_stroke (cr);

    cairo_select_font_face (cr, "Sans", CAIRO_FONT_SLANT_ITALIC,
                                        CAIRO_FONT_WEIGHT_BOLD);

	cairo_set_font_size (cr, 15);
	cairo_move_to (cr, width-60, height-5);
	cairo_show_text (cr, "Input");

	ChannelResponse *c=ti3->FirstChannel();
	while(c)
	{
		ChannelResponse *c2=NULL;
		while((c2=c->FindSubChannel(c2)))
		{
			DeviceNColorant *col=(*ti3)[c2->GetChannelIndex()];
			cairo_set_source_rgb (cr, col->red/255.0,col->green/255.0,col->blue/255.0);

			int sc=c->GetSampleCount();
			ChannelResponseSample *sam=c2->GetSamples();
			Interpolate_Lab *labint=c2->GetLabInterp_Smoothed();
			LabValue origin(sam[0]);
			double dL=(*labint)[0.0].L-(*labint)[100.0].L;

			bool first=true;

			for(int i=1;i<sc-2;++i)
			{
				double ci=sam[i+1].input;
				double pi=sam[i-1].input;
				LabValue prev=(*labint)[sam[i-1].input];
				LabValue current=(*labint)[sam[i+1].input];

				double d3=prev.L-current.L;
				double di=lin.ForwardGamma(ci/100.0)-lin.ForwardGamma(pi/100.0);
				double dLbydi=(d3/di)/dL;

				if(first)
				{
					cairo_move_to(cr,(i*width)/sc,height-(dLbydi*height)/10.0-1);
					first=false;
				}
				else
					cairo_line_to(cr,(i*width)/sc,height-(dLbydi*height)/10.0-1);
				cairo_rel_line_to(cr,2,2);
				cairo_rel_move_to(cr,0,-2);
				cairo_rel_line_to(cr,-2,2);
				cairo_line_to(cr,(i*width)/sc,height-(dLbydi*height)/10.0-1);
			}
			cairo_stroke(cr);
		}
		c=c->NextChannel();
	}
	cairo_destroy (cr);
}




// Individual channel ink limit "step" - self contained class which builds and links all
// the widgets required.  Use the "widget" member to obtain a GtkWidget which can
// be packed into a window


StepWidget_SubChannel::StepWidget_SubChannel(ProjectGUI *gui) : StepWidget(gui), chans(NULL), payload(NULL), labeltext(NULL)
{
	char *fn=gui->GetTempRoot("RawLin",".ti3");
	try
	{
		chans=new RawChannelResponses(fn,gui->gprinter.linearization.FindFloat("BaseGamma"));
	}
	catch (const char *err)
	{
		cerr << "Couldn't get ChannelResponses - chart probably hasn't been read yet!" << endl;
	}
	free(fn);

	widget=gtk_vbox_new(FALSE,0);

	if(!chans)
		return;		// Bail out if there's no .ti3 file yet.

	payload=new SubChannel_Payload(*this);

	GtkWidget *hbox=gtk_hbox_new(FALSE,8);
	gtk_widget_show(hbox);

	notebook=gtk_notebook_new();
	gtk_box_pack_start(GTK_BOX(hbox),notebook,TRUE,TRUE,0);
	gtk_widget_show(GTK_WIDGET(notebook));


//	Subchannel ratio graph

	GtkWidget *label=gtk_label_new(_("Input vs Subchannel Ratios"));
	gtk_widget_show(label);

	GtkWidget *canvas = gtk_drawing_area_new();
	gtk_widget_set_size_request(canvas, 50,50);
	g_signal_connect(G_OBJECT (canvas), "expose-event",
	                G_CALLBACK (paint_input_vs_subchannel_ratio),payload);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),GTK_WIDGET(canvas),GTK_WIDGET(label));
	gtk_widget_show(canvas);


//	dL/d_input graph

	label=gtk_label_new(_("Input vs dL/d_input"));
	gtk_widget_show(label);

	canvas = gtk_drawing_area_new();
	gtk_widget_set_size_request(canvas, 50,50);
	g_signal_connect(G_OBJECT (canvas), "expose-event",
	                G_CALLBACK (paint_input_vs_dLbydi),payload);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),GTK_WIDGET(canvas),GTK_WIDGET(label));
	gtk_widget_show(canvas);


//  Ratios

	GtkWidget *vbox=gtk_vbox_new(FALSE,0);
	gtk_box_pack_start(GTK_BOX(hbox),vbox,FALSE,FALSE,8);
	gtk_widget_show(vbox);

	GtkWidget *table=gtk_table_new(1,1,FALSE);
	gtk_table_set_col_spacings(GTK_TABLE(table),4);
	gtk_table_set_row_spacings(GTK_TABLE(table),12);
	gtk_box_pack_start(GTK_BOX(vbox),table,FALSE,FALSE,8);

	GtkWidget *tmp;

	int row=0;
	label=gtk_label_new(_("Subchannel ratios\nat inklimit"));
	gtk_table_attach_defaults(GTK_TABLE(table),label,0,2,row,row+1);
	gtk_widget_show(label);
	++row;

	ChannelResponse *c1=chans->FirstChannel();
	while(c1)
	{
		ChannelResponse *c2=NULL;
		while((c2=c1->FindSubChannel(c2)))
		{
			const char *fmtstring="%s : %s ratio:";
			labeltext=(char *)malloc(strlen(c1->GetShortName())+strlen(c2->GetShortName())+strlen(fmtstring));
			sprintf(labeltext,fmtstring,c1->GetShortName(),c2->GetShortName());
			label=gtk_label_new(labeltext);
			cerr << "Using labeltext: " << labeltext << endl;
			gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,row,row+1);
			gtk_widget_show(label);

			double ratio=payload->FindSubChannelRatio(c2->GetInkLimit(),*c1,*c2,gui->gprinter.linearization);
			cerr << "Ratio: " << ratio << endl;
			char labeltext2[6];
			sprintf(labeltext2,"%1.3lf",ratio);
			tmp=gtk_label_new(labeltext2);
			gtk_table_attach_defaults(GTK_TABLE(table),tmp,1,2,row,row+1);
			gtk_widget_show(tmp);

			++row;
		}

		c1=c1->NextChannel();
	}
	gtk_widget_show(table);

	gtk_box_pack_start(GTK_BOX(widget),hbox,TRUE,TRUE,0);
}


StepWidget_SubChannel::~StepWidget_SubChannel()
{
	if(chans)
		delete chans;
	if(payload)
		delete payload;
	if(labeltext)
		free(labeltext);
}


// Text for the Directions box

const char *StepWidget_SubChannel::GetDirections()
{
	if(chans)
	{
		return(_("This page is used to determine ratios between light and dark variants of same colour.\n"
				 "This is only applicable when using Raw Printer Channels mode, not Composite CMYK mode."));
	}
	else
	{
		return(_("Unable to open the .ti3 file, which will be created when reading the linearization chart.\n\n"
				 "One this chart has been read, this page can be used to determine ratios between light and "
				 "dark variants of the same colour.\n"
				 "This is only applicable when using Raw Printer Channels mode, not Composite CMYK mode."));
	}
}


void StepWidget_SubChannel::SaveChanges()
{
	if(chans)
	{
		int i=0;
	}
		
	gui->Save();

	modified=false;
}


// Callback function to mark the step as modified...
void StepWidget_SubChannel::markmodified(GtkWidget *wid,gpointer data)
{
	StepWidget_SubChannel *step=(StepWidget_SubChannel *)data;
	step->modified=true;
}


