#include <iostream>
#include <math.h>

#include <gtk/gtk.h>
#include <cairo.h>

#include "imagesource_devicen_preview.h"
#include "imagesource_modifiedgamma.h"
#include "linproject.h"
#include "channelresponse.h"

using namespace std;


ChannelResponse *FindSubChannel(RawChannelResponses &rcr,ChannelResponse &chan,ChannelResponse *subchan=NULL)
{
	Interpolate_Lab *li1=chan.GetLabInterp();
	if(subchan)
		subchan=subchan->NextChannel();
	else
		subchan=rcr.FirstChannel();

	while(subchan)
	{
		Interpolate_Lab *li2=subchan->GetLabInterp();
		LabValue lsub=(*li2)[50];
		if(lsub.L>(*li1)[50].L)
		{
			double l=li1->ReverseInterpolate_L(lsub.L);
			LabValue lchan=(*li1)[l];

			double de=lsub.dE(lchan);
			if(de<15.0)
				return(subchan);
		}
		subchan=subchan->NextChannel();
	}
	return(NULL);
}


double FindSubChannelRatio(double input, ChannelResponse &chan,ChannelResponse &subchan,Linearization &lin)
{
	double result=0.0;

	Interpolate_Lab *li1=chan.GetLabInterp();
	Interpolate_Lab *li2=subchan.GetLabInterp();

	LabValue l2=(*li2)[input];
//	cerr << "L* at " << input << " in " << subchan.GetLongName() << " is " << l2.L << endl;

	double level=li1->ReverseInterpolate_L(l2.L);
//	cerr << "Matching level in " << chan.GetLongName() << " found at " << level << endl;

	// Find raw device values for inklimit in subchannel and corresponding level in main channel
	double corr1=lin.ForwardGamma(input/100.0);
	double corr2=lin.ForwardGamma(level/100.0);
	result=corr2/corr1;

	return(result);
}


class Payload
{
	public:
	Payload(RawChannelResponses &ti3,Linearization &lin) : ti3(ti3), lin(lin)
	{
	}
	~Payload()
	{
	}
	RawChannelResponses &ti3;
	Linearization &lin;
};


static void paint_input_vs_subchannel_ratio (GtkWidget *widget,GdkEventExpose *eev,gpointer userdata)
{
	Payload *payload=(Payload *)userdata;
	ChannelResponseSample *paper=payload->ti3.GetPaper();
	LabValue paperlab(*paper);

	int width  = widget->allocation.width;
	int height = widget->allocation.height;

	cairo_t *cr = gdk_cairo_create (widget->window);
 
	cairo_set_source_rgb (cr, 1,1,1);
	cairo_paint (cr);

	cairo_set_source_rgb (cr, 0,0,0);
	cairo_set_line_width(cr,1.0);
	cairo_move_to (cr, 1,0);
	cairo_line_to(cr,1,height);
	cairo_move_to(cr,0,height-1);
	cairo_line_to (cr, width,height-1);
	cairo_stroke (cr);

    cairo_select_font_face (cr, "Sans", CAIRO_FONT_SLANT_ITALIC,
                                        CAIRO_FONT_WEIGHT_BOLD);
	cairo_set_font_size (cr, 9);
	cairo_move_to (cr, width-75, height-3);
	cairo_show_text (cr, "Input 100%");

	ChannelResponse *c=payload->ti3.FirstChannel();
	while(c)
	{
		ChannelResponse *c2=NULL;
		while((c2=c->FindSubChannel(c2)))
		{
			cerr << c2->GetLongName() << " is a subchannel of " << c->GetLongName() << endl;
			DeviceNColorant *col=payload->ti3[c->GetChannelIndex()];
			cairo_set_source_rgb (cr, col->red/255.0,col->green/255.0,col->blue/255.0);

			bool first=true;

			for(int i=1;i<100;++i)
			{
				float input=i;
				double ratio=FindSubChannelRatio(input,*c,*c2,payload->lin);

				if(first)
					cairo_move_to(cr,(input*width)/100.0,height-(ratio*height));
				else
					cairo_line_to(cr,(input*width)/100.0,height-(ratio*height));
				cairo_rel_line_to(cr,2,2);
				cairo_rel_move_to(cr,0,-2);
				cairo_rel_line_to(cr,-2,2);
				cairo_move_to(cr,(input*width)/100.0,height-(ratio*height));
				first=false;
			}
			cairo_stroke(cr);
		}
		c=c->NextChannel();
	}
	cairo_destroy (cr);
}


#define DEFAULT_WIDTH 640
#define DEFAULT_HEIGHT 480


int main(int argc, char *argv[])
{
//	stp_init();
	gtk_init(&argc, &argv);
	if(argc>1)
	{
		try
		{
			LinearizationCore core;
			LinProject project(core,argv[1]);
			project.Load();

			char *fn=project.GetTempRoot("RawLin",".ti3");
			RawChannelResponses ti3(fn,project.gprinter.linearization.FindFloat("BaseGamma"));
			free(fn);

			Payload payload(ti3,project.gprinter.linearization);

			GtkWidget *window;
			GtkWidget *canvas;

			window   = gtk_window_new(GTK_WINDOW_TOPLEVEL);
			g_signal_connect(G_OBJECT (window), "delete-event",
			                G_CALLBACK (gtk_main_quit), NULL);

			GtkWidget *notebook=gtk_notebook_new();
			gtk_container_add(GTK_CONTAINER(window),notebook);
			gtk_widget_show(GTK_WIDGET(notebook));

// Subchannel

			GtkWidget *label=gtk_label_new("Subchannels");
			gtk_widget_show(label);

			GtkWidget *vbox=gtk_vbox_new(TRUE,0);

			gtk_notebook_append_page(GTK_NOTEBOOK(notebook),GTK_WIDGET(vbox),GTK_WIDGET(label));
			gtk_widget_show(vbox);


			canvas = gtk_drawing_area_new();
			gtk_widget_set_size_request(canvas, DEFAULT_WIDTH, DEFAULT_HEIGHT);
			g_signal_connect(G_OBJECT (canvas), "expose-event",
			                G_CALLBACK (paint_input_vs_subchannel_ratio), &payload);
			gtk_box_pack_start(GTK_BOX(vbox),canvas,TRUE,TRUE,0);

			gtk_widget_show_all(window);

			gtk_main ();
		}
		catch(const char *err)
		{
			cerr << "Error: " << err << endl;
		}
	}
	return(0);
}


