#include <iostream>
#include <math.h>

#include <gtk/gtk.h>

#include "support/binarysearch.h"

#include "imagesource_devicen_preview.h"
#include "imagesource_modifiedgamma.h"
#include "linproject.h"
#include "channelresponse.h"
#include "labinterp.h"

using namespace std;


class BinarySearch_HueSlope : public BinarySearch
{
	public:
	BinarySearch_HueSlope(Interpolate_Lab &lab) : BinarySearch(), labint(lab)
	{
		origin=labint[0.0];
	}
	virtual ~BinarySearch_HueSlope()
	{
	}
	virtual double Evaluate(double input)
	{
		double pi=input-1.0;
		double ci=input+1.0;
		LabValue current=labint[ci];
		LabValue prev=labint[pi];

		double d1=prev.dE(origin);
		double d2=current.dE(origin);
		double d3=current.dE(prev);
		double slope=(d2-d1)/d3;
		return(slope);
	}
	protected:
	Interpolate_Lab &labint;
	LabValue origin;
};


class BinarySearch_dEbydi : public BinarySearch
{
	public:
	BinarySearch_dEbydi(Interpolate_Lab &lab,double gamma) : BinarySearch(), labint(lab), gamma(gamma)
	{
		cerr << "Gamma: " << gamma << endl;
		origin=labint[0.0];
	}
	virtual ~BinarySearch_dEbydi()
	{
	}
	virtual double Evaluate(double input)
	{
		double pi=input-1.0;
		double ci=input+1.0;
		LabValue current=labint[ci];
		LabValue prev=labint[pi];

		double d3=current.dE(prev);
		double di=ForwardGamma(ci/100.0)-ForwardGamma(pi/100.0);
		double dEbydi=d3/di;
		return(dEbydi);
	}
	double ForwardGamma(double in)
	{
		return(1.0-ImageSource_ModifiedGamma::ModifiedGamma(1.0-in,gamma));
	}
	protected:
	Interpolate_Lab &labint;
	double gamma;
	LabValue origin;
};


class BinarySearch_dLbydi : public BinarySearch
{
	public:
	BinarySearch_dLbydi(Interpolate_Lab &lab,double gamma) : BinarySearch(), labint(lab), gamma(gamma)
	{
		cerr << "Gamma: " << gamma << endl;
		origin=labint[0.0];
	}
	virtual ~BinarySearch_dLbydi()
	{
	}
	virtual double Evaluate(double input)
	{
		double pi=input-1.0;
		double ci=input+1.0;
		LabValue current=labint[ci];
		LabValue prev=labint[pi];

		double d3=prev.L-current.L;
		double di=ForwardGamma(ci/100.0)-ForwardGamma(pi/100.0);
		double dEbydi=d3/di;
		return(dEbydi);
	}
	double ForwardGamma(double in)
	{
		return(1.0-ImageSource_ModifiedGamma::ModifiedGamma(1.0-in,gamma));
	}
	protected:
	Interpolate_Lab &labint;
	double gamma;
	LabValue origin;
};


void AnalyzeChannels(Linearization &lin,RawChannelResponses &chans)
{
	double dLbyditarget=100.0;

	ChannelResponse *c=chans.FirstChannel();
	while(c)
	{
		if(c->GetType()!=CHANNELRESPONSETYPE_BLACK)
		{
			cerr << "Channel: " << c->GetLongName() << endl;
			Interpolate_Lab *labint=c->GetLabInterp();
			BinarySearch_HueSlope hueslope(*labint);

			double result=hueslope.Search(1.0,99.0,0.975);

			cerr << "Ink limit: " << result << endl;
			BinarySearch_dLbydi dlbydi(*labint,lin.FindFloat("BaseGamma"));
			double dL=dlbydi.Evaluate(result);

			double range=(*labint)[0.0].L-(*labint)[100.0].L;
			double scaled=dL/range;
			if(c->GetType()!=CHANNELRESPONSETYPE_YELLOW && scaled<dLbyditarget)
				dLbyditarget=scaled;

			cerr << "Total dL for channel: " << range << endl;

			cerr << "dL by di at ink limit: " << dL << endl << endl;
		}
		c=c->NextChannel();
	}

	c=chans.FirstChannel();
	while(c)
	{
		if(c->GetType()==CHANNELRESPONSETYPE_BLACK)
		{
			cerr << "Channel: " << c->GetLongName() << endl;
			Interpolate_Lab *labint=c->GetLabInterp();

			double range=(*labint)[0.0].L-(*labint)[100.0].L;
			cerr << "Scaled dL by di target: " << range * dLbyditarget << endl;
			BinarySearch_dLbydi dlbydi(*labint,lin.FindFloat("BaseGamma"));

			double result=dlbydi.Search(1.0,99.0,range*dLbyditarget);
			cerr << "Ink limit: " << result << endl;
		}
		c=c->NextChannel();
	}
}


int main(int argc, char *argv[])
{
//	stp_init();
	gtk_init(&argc, &argv);
	if(argc>1)
	{
		try
		{
			LinearizationCore core;
			LinProject project(core,argv[1]);
			project.Load();

			char *fn=project.GetTempRoot("RawLin",".ti3");
			RawChannelResponses ti3(fn,project.gprinter.linearization.FindFloat("BaseGamma"));
			free(fn);

			AnalyzeChannels(project.gprinter.linearization,ti3);
		}
		catch (const char *err)
		{
			cerr << "Error: " << err << endl;
		}
	}
	return(0);
}

