#ifndef TI3FILE_H
#define TI3FILE_H

#include "cgats/cgats.h"
#include "imagesource/devicencolorant.h"


// Simplistic wrapper around opening a CGATS file and finding indices for various
// variables within.

class CGATSFile
{
	public:
	CGATSFile(const char *filename,const char *type=NULL);
	virtual ~CGATSFile();
	virtual void Save(const char *filename);
	virtual const char *GetFieldName(int table,int idx);
	virtual int FindField(int table,const char *name);
	virtual const char *GetKeyword(int table,const char *key);
	protected:
	cgats *cgatsfile;
};


// TI file wrapper - doesn't provide access to the actual data, but
// builds a list of colorants, accessible via the DeviceNColorantList
// interface, for use in interpreting the DeviceN TIFFs produced by printtarg
// Also serves as a base class for TI3File.

class TIXFile : public CGATSFile, public DeviceNColorantList
{
	public:
	TIXFile(const char *filename,const char *type="CTI1");
	virtual ~TIXFile();
	virtual int GetColorantIndex(const char *name);

	virtual int GetColorantCount();
	virtual int GetSets();
	virtual double **GetData(int set);
	virtual const char *GetFieldName(int idx);
	virtual int FindField(const char *name);

	// Utility function to create a mapping between a raw DeviceNColorantList and the colorants within a .tix file.
	// The array of ints which is returned must be delete[]ed when no longer required.
	virtual int *CreateRawChannelMap(DeviceNColorantList &colorants);
	protected:
	char *colorantstring;
	cgats_table *table;
	int colorants;
	int colorantoffset;	// The first field which contains a colorant
	int sets;
	int Xindex,Yindex,Zindex;	// Indices of the X, Y and Z fields, adjusted by colorantoffset
};


// TI3 file wrapper - provides access to the data from a successfully
// read chart.
// Abstracted yet further for linearization purposes by the
// RawChannelResponses class

class TI3File : public TIXFile
{
	public:
	TI3File(const char *filename);
	virtual ~TI3File();
	protected:
};

#endif

