// Gutenprint tuning class - keeps track of paramters such as dropsizes and
// subchannel ratios.  Values of interest to Gutenprint, but not to other
// apps making use of a linearization.

#include <iostream>

#include "tuning.h"

using namespace std;

ConfigTemplate Tuning::Template[]=
{
	ConfigTemplate("DropSizeLimit0",float(0.5)),	// Used to ensure the paper isn't saturated by the
	ConfigTemplate("DropSizeLimit1",float(.25)),	// dropsize calibration chart
	ConfigTemplate("DropSizeLimit2",float(0.1)),
	ConfigTemplate("DropSizeRatio0",float(1.0)),	// These are the values that will be fed to Gutenprint
	ConfigTemplate("DropSizeRatio1",float(1.0)),
	ConfigTemplate("DropSizeRatio2",float(1.0)),
	ConfigTemplate("TuningGamma",float(0.45)),
	ConfigTemplate()
};


Tuning::Tuning(ConfigFile *file) : ConfigDB(Template)
{
	new ConfigDBHandler(file,"[GPTuning]",this);
}


Tuning::~Tuning()
{

}


static const char *dropsizelimitnames[]=
{
	"DropSizeLimit0",
	"DropSizeLimit1",
	"DropSizeLimit2"
};

void Tuning::DropSizeLimitToDB(int chan,float droplimit)
{
	if(chan<3)
		SetFloat(dropsizelimitnames[chan],droplimit);
}


float Tuning::DropSizeLimitFromDB(int chan)
{
	if(chan<3)
		return(FindFloat(dropsizelimitnames[chan]));
	else
		return(0.0);
}


static const char *dropsizerationames[]=
{
	"DropSizeRatio0",
	"DropSizeRatio1",
	"DropSizeRatio2"
};

void Tuning::DropSizeRatioToDB(int chan,float dropratio)
{
	if(chan<3)
		SetFloat(dropsizerationames[chan],dropratio);
}


float Tuning::DropSizeRatioFromDB(int chan)
{
	if(chan<3)
		return(FindFloat(dropsizerationames[chan]));
	else
		return(1.0);
}


///////////////////////////////////////////////////////////////////////////////
// DropSize_Colorants class - fakes a list of colorants for the benefit of the
// DropSize calibration code


DropSize_Colorants::DropSize_Colorants(stp_vars_t *vars) : GPColorantList(vars)
{
	while(FirstColorant())
		delete FirstColorant();
	BuildList(vars);
}


DropSize_Colorants::DropSize_Colorants(GPrinterSettings *gp) : GPColorantList(gp)
{
	while(FirstColorant())
		delete FirstColorant();
	BuildList(gp->stpvars);
}


DropSize_Colorants::~DropSize_Colorants()
{

}


void DropSize_Colorants::BuildList(stp_vars_t *vars)
{
	cerr << "In DropSize_Colorants::BuildList()" << endl;
	new DeviceNColorant(*this,"Light Light Black");
	new DeviceNColorant(*this,"Light Black");
	new DeviceNColorant(*this,"Black");
	// Magenta is included merely as a spacer colour, so we include but disable it.
	new DeviceNColorant(*this,"Magenta");
	DeviceNColorant *m=(*this)[3];
	if(m)
		m->Disable();
}


